/*+
 * ExifMetadata.ms
 *
 *  Contains the Metadata subclass for Exif.
 *
 *  See method descriptions below.
 *
-*/

#include "sys:/Metadata.ms"

/* ----------------------------------------------------------------------------
 *  ExifMetadata
 *
 *  construct a new empty Metadata object.
 *
 *  validate    - true to validate on set (default).
 *
 *  example:
 *      // load metadata from image.
 *      metadata = new Metadata();
 -----------------------------------------------------------------------------*/
function ExifMetadata(validate)
{
    this._init(validate);
    this.blankDocument();
}

/* ----------------------------------------------------------------------------
 *  ExifMetadata_blank
 *
 *  construct a blank document.
 *
 *  example:
 *      // load metadata from image.
 *      metadata = new ExifMetadata();
 *      metadata.blankDocument();
 -----------------------------------------------------------------------------*/
function ExifMetadata_blank()
{
    this._clear();
    var dom = this.getDOM();

    var root = dom.createElement("Exif");
    root.setAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
    var xsdDocument = getPropertyValue("paths.SysPath") + "/Exif.xsd";
    root.setAttribute("xsi:noNamespaceSchemaLocation", xsdDocument);
    dom.appendChild(root);

    var ifd0 = dom.createElement("IFD0");
    root.appendChild(ifd0);
    this.setVersion(2.2);
    if (this.doValidation)
        this.validate();
}

/* ----------------------------------------------------------------------------
 *  ExifMetadata_loadFromFile
 *
 *  load ExifMetadata object from the file.
 *
 *  fileName    - (string) the name of the file to get the data from
 *
 *  example:
 *      // load metadata from image.
 *      metadata = new ExifMetadata();
 *      metadata.loadFromFile("image.jpg");
 -----------------------------------------------------------------------------*/
function ExifMetadata_loadFromFile(fileName)
{
    this._loadFromFile(fileName, "Exif");
}

/* ----------------------------------------------------------------------------
 *  ExifMetadata_loadFromMedia
 *
 *  load ExifMetadata object from the media.
 *
 *  media       - (string) the name of the file to get the data from
 *
 *  example:
 *      // load metadata from image.
 *      var m = new Media();
 *      m.load(name @ "image.jpg");
 *      metadata = new Metadata();
 *      metadata.loadFromMedia(m);
 -----------------------------------------------------------------------------*/
function ExifMetadata_loadFromMedia(media)
{
    this._loadFromFile(media, "Exif");
}

Exif_ResolutionUnit = [
    { key : 1, value : "none" },
    { key : 2, value : "inches" },
    { key : 3, value : "centimeters" }
];

Exif_ColorSpace = [
    { key : 1, value : "sRgb" },
    { key : 65535, value : "uncalibrated" },
];

Exif_Orientation = [
    {key : 1, value : "top-left"},
    {key : 2, value : "top-right"},
    {key : 3, value : "bottom-right"},
    {key : 4, value : "bottom-left"},
    {key : 5, value : "left-top"},
    {key : 6, value : "right-top"},
    {key : 7, value : "right-bottom"},
    {key : 8, value : "left-bottom"}
];

Exif_Compression =  [
    {key : 1, value : "uncompressed"},
    {key : 2, value : "CCITT modified Huffman RLE"},
    {key : 3, value : "CCITT T.4"},
    {key : 4, value : "CCITT T.6"},
    {key : 5, value : "Lempel-Ziv-Welch"},
    {key : 6, value : "JPEG"},
    {key : 7, value : "JPEG DCT"},

    {key : 8, value : "Adobe deflate"},
    {key : 32766, value : "NeXT 2-bit RLE"},
    {key : 32771, value : "word aligned"},
    {key : 32773, value : "Macintosh RLE"},
    {key : 32809, value : "Thunderscan RLE"},
    {key : 32895, value : "IT8 CT w/padding"},
    {key : 32896, value : "IT8 Linework RLE"},
    {key : 32897, value : "IT8 Monochrome picture"},
    {key : 32898, value : "IT8 Binary line art"},
    {key : 32908, value : "Pixar companded 10bit LZW"},
    {key : 32909, value : "Pixar companded 11bit LZW"},
    {key : 32946, value : "Deflate"},
    {key : 32947, value : "Kodak DCS encoding"},
    {key : 34661, value : "ISO JBIG"},
    {key : 34676, value : "SGI Log Luminance RLE"},
    {key : 34677, value : "SGI Log 24-bit packed"},
    {key : 34712, value : "Leadtools JPEG2000"}
];

Exif_PhotometricInterpretation = [
    {key :  0, value : "Monochrome(Min=white)"},
    {key :  1, value : "Monochrome"},
    {key :  2, value : "RGB"},
    {key :  3, value : "Indexed"},
    {key :  4, value : "Mask"},
    {key :  5, value : "Separated"},
    {key :  6, value : "YCbCr"},
    {key :  8, value : "CIELab"},
    {key :  9, value : "ICCLab"},
    {key : 10, value : "ITULab"}
];

Exif_PlanarConfiguration = [
    {key : 1, value : "chunky"},
    {key : 2, value : "planar"}
];

Exif_YCbCrPositioning = [
    {key : 1, value : "center"},
    {key : 2, value : "co-sited"}
];

Exif_ExposureProgram = [
    {key : 0, value : "Not defined"},
    {key : 1, value : "Manual"},
    {key : 2, value : "Normal program"},
    {key : 3, value : "Aperture priority"},
    {key : 4, value : "Shutter priority"},
    {key : 5, value : "Creative program"},
    {key : 6, value : "Action program"},
    {key : 7, value : "Portrait mode"},
    {key : 8, value : "Landscape mode"}
];

Exif_MeteringMode = [
    {key :   0, value : "unknown"},
    {key :   1, value : "Average"},
    {key :   2, value : "CenterWeightedAverage"},
    {key :   3, value : "Spot"},
    {key :   4, value : "MultiSpot"},
    {key :   5, value : "Pattern"},
    {key :   6, value : "Partial"},
    {key : 255, value : "other"}
];

Exif_LightSource = [
    {key :   0, value : "unknown"},
    {key :   1, value : "Daylight"},
    {key :   2, value : "Fluorescent"},
    {key :   3, value : "Tungsten"},
    {key :  17, value : "Standard light A"},
    {key :  18, value : "Standard light B"},
    {key :  19, value : "Standard light C"},
    {key :  20, value : "D55"},
    {key :  21, value : "D65"},
    {key :  22, value : "D75"},
    {key : 255, value : "other"}
];

Exif_SensingMethod = [
    {key : 1, value : "Not defined"},
    {key : 2, value : "One-chip color area sensor"},
    {key : 3, value : "Two-chip color area sensor"},
    {key : 4, value : "Three-chip color area sensor"},
    {key : 5, value : "Color sequential area sensor"},
    {key : 7, value : "Trilinear sensor"},
    {key : 8, value : "Color sequential linear sensor"}
];

Exif_FileSource = [
    {key : 3, value : "DSC"}
];

Exif_SceneType = [
    {key : 1, value : "A directly photographed image"}
];

Exif_GPSMeasure = [
	{key : 2, value : "2-D"},
	{key : 3, value : "3-D"}
];

Exif_GPSCorrection = [
	{key : 0, value : "No Correction"},
	{key : 1, value : "Corrected"}
];


function Exif_Lookup(value, lookupArr)
{
    if (value == null || value == "")
        return null;
    var key = parseInt(value);
    for (var i in lookupArr)
    {
        if (lookupArr[i].key == key)
            return lookupArr[i].value;
    }

    return "Unknown";
}

function Exif_Unlookup(value, lookupArr)
{
    for (i in lookupArr)
    {
        if (lookupArr[i].value.toLowerCase() == value.toLowerCase())
            return lookupArr[i].key;
    }

    return -1;
}

function Exif_getImageDescription()
{
    return this.getValue("IFD0/ImageDescription");
}

function Exif_setImageDescription(value)
{
    this.setValue("IFD0/ImageDescription", value);
    if (this.doValidation)
        this.validate();
}

function Exif_getMake()
{
    return this.getValue("IFD0/Make");
}

function Exif_setMake(value)
{
    return this.setValue("IFD0/Make", value);
}

function Exif_getModel()
{
    return this.getValue("IFD0/Model");
}

function Exif_setModel(value)
{
    this.setValue("IFD0/Model", value);
    if (this.doValidation)
        this.validate();
}

function Exif_getOrientation()
{
    return Exif_Lookup(this.getValue("IFD0/Orientation"), Exif_Orientation);
}

function Exif_setOrientation(value)
{
    this.setValue("IFD0/Orientation", Exif_Unlookup(value, Exif_Orientation));
    if (this.doValidation)
        this.validate();
}

function Exif_getSoftware()
{
    return this.getValue("IFD0/Software");
}

function Exif_setSoftware(value)
{
    this.setValue("IFD0/Software", value);
    if (this.doValidation)
        this.validate();
}

function Exif_getArtist()
{
    return this.getValue("IFD0/Artist");
}

function Exif_setArtist(value)
{
    this.setValue("IFD0/Artist", value);
    if (this.doValidation)
        this.validate();
}

function Exif_padFront(val, count)
{
    var s = val.toString();
    while(s.length < count)
        s = "0" + s;
    return s;
}

function Exif_dateToString(date)
{
    if (date == null)
        date = new Date();

    if (typeof(date) == "object")
    {
        var yr = date.getFullYear().toString();
        var month = Exif_padFront(date.getMonth() + 1, 2);
        var day = Exif_padFront(date.getDate(), 2);
        var hrs = Exif_padFront(date.getHours(), 2);
        var mins = Exif_padFront(date.getMinutes(), 2);
        var secs = Exif_padFront(date.getSeconds(), 2);
        //var ofs = Exif_padFront(date.getTimezoneOffset(), 2);
        return yr + ":" + month + ":" + day + " " + hrs + ":" + mins + ":" + secs;
    }

    return date;
}

function Exif_parseDate(dateStr)
{
    if (dateStr == null)
        return dateStr;
    var strs = dateStr.split(" ");
    if (strs.length != 2)
    {
        return dateStr;    //  dont parse
    }

    var date = strs[0].split(":");
    var time = strs[1].split(":");

    if (date.length != 3 || time.length != 3)
        return dateStr;

    var year = parseInt(date[0], 10);
    var month = parseInt(date[1], 10) - 1;
    var day = parseInt(date[2], 10);

    var hr = parseInt(time[0], 10);
    var min = parseInt(time[1], 10);
    var sec = parseInt(time[2], 10);

    //print("\t" + time + "," + dateStr + "\n");
    //print("\t" + hr + ":" + min + ":" + sec + "\n");
    return new Date(year, month, day, hr, min, sec, 0);
}

function Exif_getDateTime()
{
    return Exif_parseDate(this.getValue("IFD0/DateTime"));
}

function Exif_setDateTime(value)
{
    this.setValue("IFD0/DateTime", Exif_dateToString(value));
    if (this.doValidation)
        this.validate();
}

function Exif_getEditorCopyright()
{
    return this.getValue("IFD0/Copyright/Editor");
}

function Exif_setEditorCopyright(value)
{
    var obj = this.getValue("IFD0/Copyright");
    var photographer = " ";

    if (obj != null)
    {
        photographer = obj.Photographer;
    }

    this.setValue("IFD0/Copyright", "");
    this.setValue("IFD0/Copyright/Photographer", photographer);
    this.setValue("IFD0/Copyright/Editor", value);

    if (this.doValidation)
        this.validate();
}

function Exif_getPhotographerCopyright()
{
    return this.getValue("IFD0/Copyright/Photographer");
}

function Exif_setPhotographerCopyright(value)
{
    var obj = this.getValue("IFD0/Copyright");
    var editor = null;
    if (obj != null)
    {
        editor = obj.Editor;
    }

    this.setValue("IFD0/Copyright", "");
    this.setValue("IFD0/Copyright/Photographer", value);
    if (editor != null)
        this.setValue("IFD0/Copyright/Editor", editor);

    if (this.doValidation)
        this.validate();
}

function Exif_getImageWidth()
{
    var val = this.getValue("IFD0/ImageWidth");
    if (val == null)
        return this.getPixelXDimension();
    else
        return val;
}

function Exif_getImageLength()
{
    var val = this.getValue("IFD0/ImageLength");
    if (val == null)
        return this.getPixelYDimension();
    else
        return val;
}

function Exif_getBitsPerSample()
{
    return this.getValue("IFD0/BitsPerSample");
}

function Exif_getCompression()
{
    return Exif_Lookup(this.getValue("IFD0/Compression"),
                Exif_Compression);
}

function Exif_getPhotometricInterpretation()
{
    return Exif_Lookup(this.getValue("IFD0/PhotometricInterpretation"),
                Exif_PhotometricInterpretation);
}

function Exif_getPlanarConfiguration()
{
    return Exif_Lookup(this.getValue("IFD0/PlanarConfiguration"),
                Exif_PlanarConfiguration);
}

function Exif_getYCbCrSubSampling()
{
    return this.getValue("IFD0/YCbCrSubSampling");
}

function Exif_getYCbCrPositioning()
{
    return Exif_Lookup(this.getValue("IFD0/YCbCrPositioning"),
                Exif_YCbCrPositioning);
}

function Exif_getXResolution()
{
    return this.getValue("IFD0/XResolution");
}

function Exif_getYResolution()
{
    return this.getValue("IFD0/YResolution");
}

function Exif_getResolutionUnit()
{
    return Exif_Lookup(this.getValue("IFD0/ResolutionUnit"),
                Exif_ResolutionUnit);
}

function Exif_getWhitePoint()
{
    return this.getValue("IFD0/WhitePoint");
}

function Exif_getPrimaryChromaticities()
{
    return this.getValue("IFD0/PrimaryChromaticities");
}

function Exif_getYCbCrCoefficients()
{
    return this.getValue("IFD0/YCbCrCoefficients");
}

function Exif_getReferenceBlackWhite()
{
    return this.getValue("IFD0/ReferenceBlackWhite");
}

function Exif_toUserVersion(str)
{
    if (str == null)
        return str;
    if (str.length != 4)
        error("Invalid Exif version string: " + str);

    return str.substr(0, 2) + "." + str.substr(2,2);
}

function Exif_fromUserVersion(versionString)
{
    var vers = versionString.toString().split(".");
    if (vers.length > 2)
        error("invalid Exif version string: " + versionString);
    if (vers.length == 0)
        return "0000";
    if (vers.length == 1)
        vers = vers.concat("00");

    if (vers[0].length > 2)
        error("major version for exif must be between 0 and 99");
    while (vers[0].length < 2)
        vers[0] = "0" + vers[0];
    if (vers[1].length > 2)
        error("minor version for exif must be between 0 and 99");
    while (vers[1].length < 2)
        vers[1] = vers[1] + "0";
    return vers[0] + vers[1];
}


function Exif_getVersion()
{
    return Exif_toUserVersion(this.getValue("IFDExif/ExifVersion"));
}

function Exif_setVersion(versionString)
{
    this.setValue("IFDExif/ExifVersion", Exif_fromUserVersion(versionString));
    if (this.doValidation)
        this.validate();
}

function Exif_getFlashPixVersion()
{
    return Exif_toUserVersion(this.getValue("IFDExif/FlashPixVersion"));
}

function Exif_setFlashPixVersion(versionString)
{
    this.setValue("IFDExif/FlashPixVersion", Exif_fromUserVersion(versionString));
    if (this.doValidation)
        this.validate();
}

function Exif_getColorspace()
{
    return Exif_Lookup(this.getValue("IFDExif/Colorspace"), Exif_ColorSpace);
}

function Exif_getPixelXDimension()
{
    return this.getValue("IFDExif/PixelYDimension");
}

function Exif_getPixelYDimension()
{
    return this.getValue("IFDExif/PixelYDimension");
}

function Exif_getComponentsConfiguration()
{
    return this.getValue("IFDExif/ComponentsConfiguration");
}

function Exif_getCompressedBitsPerPixel()
{
    return this.getValue("IFDExif/CompressedBitsPerPixel");
}

function Exif_getUserComment()
{
    return this.getValue("IFDExif/UserComment");
}

function Exif_setUserComment(value)
{
    this.setValue("IFDExif/UserComment", value);
    if (this.doValidation)
        this.validate();
}

function Exif_getRelatedSoundFile()
{
    return this.getValue("IFDExif/RelatedSoundFile");
}

function Exif_getDateTimeOriginal()
{
    return Exif_parseDate(this.getValue("IFDExif/DateTimeOriginal"));
}

function Exif_getDateTimeDigitized()
{
    return Exif_parseDate(this.getValue("IFDExif/DateTimeDigitized"));
}

function Exif_getSubSecTime()
{
    return this.getValue("IFDExif/SubSecTime");
}

function Exif_getSubSecTimeOriginal()
{
    return this.getValue("IFDExif/SubSecTimeOriginal");
}

function Exif_getSubSecTimeDigitized()
{
    return this.getValue("IFDExif/SubSecTimeDigitized");
}

function Exif_getExposureTime()
{
    return this.getValue("IFDExif/ExposureTime");
}

function Exif_computeShutterSpeedValue(val)
{
    if (val == null)
    	return null;
    var parts = val.split("/");
    if (parts.length == 2 && parts[1] != 0)
        return Math.round(10000 * Math.pow(2, -parts[0] / parts[1])) / 10000;
    return val;
}

function Exif_getShutterSpeedValue()
{
    return Exif_computeShutterSpeedValue(this.getValue("IFDExif/ShutterSpeedValue"));
}

function Exif_computeFNumber(avFract)
{
    if (avFract == null)
        return null;
    var parts = avFract.split("/");
    if (parts.length == 2 && parts[1] != 0)
    {
        var av = parts[0] / parts[1];
        return "F" + Math.round( 1000 * Math.pow(2, av / 2.0) ) / 1000;
    }
    return avFract;
}

function Exif_resolveFraction(fract, digs)
{
    if (fract == null  ||  digs == null)
        return null;
    var parts = fract.split("/");
    if (parts.length == 2)
    {
        if (parts[1] == 0)
            return null;	// DMP: changed this from "0.0" because values with 0 denom's are supposed to be undefined values
        var scale = Math.pow(10, digs);
        return Math.round(scale * parts[0] / parts[1]) / scale;
    }
    return fract;
}

function Exif_getApertureValue()
{
    return Exif_computeFNumber(this.getValue("IFDExif/ApertureValue"));
}

function Exif_getBrightnessValue()
{
    return this.getValue("IFDExif/BrightnessValue");
}

function Exif_getExposureBiasValue()
{
    return this.getValue("IFDExif/ExposureBiasValue");
}

function Exif_getMaxApertureValue()
{
    return Exif_computeFNumber(this.getValue("IFDExif/MaxApertureValue"));
}

function Exif_getSubjectDistance()
{
    return Exif_resolveFraction(this.getValue("IFDExif/SubjectDistance"), 3);
}

function Exif_getMeteringMode()
{
    return Exif_Lookup(this.getValue("IFDExif/MeteringMode"),
                Exif_MeteringMode);
}

function Exif_getLightSource()
{
    return Exif_Lookup(this.getValue("IFDExif/LightSource"),
                Exif_LightSource);
}

function Exif_getFlash()
{
    return this.getValue("IFDExif/Flash");
}

function Exif_getFocalLength()
{
    return Exif_resolveFraction(this.getValue("IFDExif/FocalLength"), 3);
}

function Exif_getFNumber()
{
    return Exif_resolveFraction(this.getValue("IFDExif/FNumber"), 3);
}

function Exif_getExposureProgram()
{
    return Exif_Lookup(this.getValue("IFDExif/ExposureProgram"),
                Exif_ExposureProgram);
}

function Exif_getSpectralSensitivity()
{
    return this.getValue("IFDExif/SpectralSensitivity");
}

function Exif_getISOSpeedRatings()
{
    return this.getValue("IFDExif/ISOSpeedRatings");
}

function Exif_getOECF()
{
    return this.getValue("IFDExif/OECF");
}

function Exif_getFlashEnergy()
{
    return this.getValue("IFDExif/FlashEnergy");
}

function Exif_getSpatialFrequencyResponse()
{
    return this.getValue("IFDExif/SpatialFrequencyResponse");
}

function Exif_getFocalPlaneXResolution()
{
    return this.getValue("IFDExif/FocalPlaneXResolution");
}

function Exif_getFocalPlaneYResolution()
{
    return this.getValue("IFDExif/FocalPlaneYResolution");
}

function Exif_getFocalPlaneResolutionUnit()
{
    return Exif_Lookup(this.getValue("IFDExif/FocalPlaneResolutionUnit"),
                Exif_ResolutionUnit);
}

function Exif_getSubjectLocation()
{
    return this.getValue("IFDExif/SubjectLocation");
}

function Exif_getExposureIndex()
{
    return this.getValue("IFDExif/ExposureIndex");
}

function Exif_getSensingMethod()
{
    return Exif_Lookup(this.getValue("IFDExif/SensingMethod"),
                Exif_SensingMethod);
}

function Exif_getFileSource()
{
    return Exif_Lookup(this.getValue("IFDExif/FileSource"),
                Exif_FileSource);
}

function Exif_getSceneType()
{
    return Exif_Lookup(this.getValue("IFDExif/SceneType"),
                Exif_SceneType);
}

function Exif_getCFAPattern()
{
    return this.getValue("IFDExif/CFAPattern");
}

function Exif_getSubjectArea()
{
    return this.getValue("IFDExif/SubjectArea");
}

function Exif_getMakerNote()
{
    return this.getValue("IFDExif/MakerNote");
}

function Exif_getCustomRendered()
{
    return this.getValue("IFDExif/CustomRendered");
}

function Exif_getExposureMode()
{
    return this.getValue("IFDExif/ExposureMode");
}

function Exif_getWhiteBalance()
{
    return this.getValue("IFDExif/WhiteBalance");
}

function Exif_getDigitalZoomRatio()
{
    return Exif_resolveFraction(this.getValue("IFDExif/DigitalZoomRatio"), 3);
}

function Exif_getFocalLengthIn35mmFilm()
{
    return this.getValue("IFDExif/FocalLengthIn35mmFilm");
}

function Exif_getSceneCaptureType()
{
    return this.getValue("IFDExif/SceneCaptureType");
}

function Exif_getGainControl()
{
    return Exif_resolveFraction(this.getValue("IFDExif/GainControl"), 3);
}

function Exif_getContrast()
{
    return this.getValue("IFDMakerNote/Contrast");
}

function Exif_getSaturation()
{
    return this.getValue("IFDMakerNote/Saturation");
}

function Exif_getSharpness()
{
    return this.getValue("IFDMakerNote/Sharpness");
}

function Exif_getDeviceSettingDescription()
{
    return this.getValue("IFDExif/DeviceSettingDescription");
}

function Exif_getSubjectDistanceRange()
{
    return this.getValue("IFDExif/SubjectDistanceRange");
}

function Exif_getImageUniqueID()
{
    return this.getValue("IFDExif/ImageUniqueID");
}

function Exif_getGPSVersionID()
{
	return this.getValue("IFDGPS/GPSVersionID");
}

function Exif_parseLocation(obj, tag)
{
	var loc = obj.getValue("IFDGPS/"+tag);
	if (loc == null)
		return null;

	// Degrees
	var rstring = "";
	var parts = loc[0].split("/");
	if (parts.length == 2)
	{
		if (parts[1] == 0)
			return null;
		rstring += (Math.round(100 * parts[0] / parts[1]) / 100).toString();
	}
	else
		rstring += loc.toString();

	// Minutes
	parts = loc[1].split("/");
	if (parts.length == 2)
	{
		if (parts[1] == 0)
			return null;
		rstring += " " + (Math.round(100 * parts[0] / parts[1]) / 100) + "'";
	}
	else
		rstring += " " + loc + "'";

	// Seconds
	parts = loc[2].split("/");
	if (parts.length == 2)
	{
		if (parts[1] == 0)
			return null;
		rstring += " " + (Math.round(100 * parts[0] / parts[1]) / 100) + '"';
	}
	else
		rstring += " " + loc + '"';

	// Reference point (NESW)
	var ref = obj.getValue("IFDGPS/"+tag+"Ref");
	if (ref == 'N')
		rstring += ' North';
	else if (ref == 'S')
		rstring += ' South';
	else if (ref == 'E')
		rstring += ' East';
	else if (ref == 'W')
		rstring +=' West';

	return rstring;
}

function Exif_getGPSLatitude()
{
	return Exif_parseLocation(this, "GPSLatitude");
}

function Exif_getGPSLongitude()
{
	return Exif_parseLocation(this, "GPSLongitude");
}

function Exif_getGPSAltitude()
{
	var alt = Exif_resolveFraction(this.getValue("IFDGPS/GPSAltitude"), 0);
	if (alt == null)
		return null;
	if (this.getValue("IFDGPS/GPSAltitudeRef") == 0)
		alt += "m Above Sea Level";
	else
		alt += "m Below Sea Level";
	return alt;
}

function Exif_getGPSTimeStamp()
{
	var time = this.getValue("IFDGPS/GPSTimeStamp");
	if (time == null)
		return null;
	var hour = time[0].split("/");
	var minute = time[1].split("/");
	var second = time[2].split("/");
	return Exif_padFront(hour[0], 2) + ":" + Exif_padFront(minute[0], 2) + ":" + Exif_padFront(second[0], 2);
}

function Exif_getGPSDateStamp()
{
	return this.getValue("IFDGPS/GPSDateStamp");
}

function Exif_getGPSFullUTC()
{
	var date = this.getValue("IFDGPS/GPSDateStamp");
	var time = this.getGPSTimeStamp();
	if (!date)
		return time;
	if (!time)
		return date;
	return date + " " + time;
}

function Exif_getGPSSatellites()
{
	return this.getValue("IFDGPS/GPSSatellites");
}

function Exif_getGPSStatus()
{
	var stat = this.getValue("IFDGPS/GPSStatus");
	if (stat == 'A')
		stat = "Active";
	else if (stat == 'V')
		stat = "Void";
	else
		stat = null;
	return stat;
}

function Exif_getGPSMeasureMode()
{
	return Exif_Lookup(this.getValue("IFDGPS/GPSMeasureMode"), Exif_GPSMeasure);
}

function Exif_getGPSDOP()
{
	return Exif_resolveFraction(this.getValue("IFDGPS/GPSDOP"), 3);
}

function Exif_getGPSSpeed()
{
	var value = Exif_resolveFraction(this.getValue("IFDGPS/GPSSpeed"), 3);
	if (value == null)
		return null;
	var units = this.getValue("IFDGPS/GPSSpeedRef");
	if (units == 'K')
		units = "km/h";
	else if (units == 'M')
		units = "mph";
	else if (units == 'N')
	{
		if (value == 1)
			units = "knot"
		else
			units = "knots";
	}
	else
		units = "";
	return value + " " + units;
}

function Exif_GPSNorthType(cm)
{
	if (cm == 'M')
		cm = "Magnetic North";
	else if (cm == 'T')
		cm = "True North";
	else
		cm= "";
	return cm;
}

function Exif_getGPSTrack()
{
	var value = Exif_resolveFraction(this.getValue("IFDGPS/GPSTrack"), 3);
	if (value == null)
		return null;
	return value + " " + Exif_GPSNorthType(this.getValue("IFDGPS/GPSTrackRef"));
}

function Exif_getGPSImgDirection()
{
	var value = Exif_resolveFraction(this.getValue("IFDGPS/GPSImgDirection"), 3);
	if (value == null)
		return null;
	return value + " " + Exif_GPSNorthType(this.getValue("IFDGPS/GPSImgDirectionRef"));
}

function Exif_getGPSMapDatum()
{
	return this.getValue("IFDGPS/GPSMapDatum");
}

function Exif_getGPSDestLatitude()
{
	return Exif_parseLocation(this, "GPSDestLatitude");
}

function Exif_getGPSDestLongitude()
{
	return Exif_parseLocation(this, "GPSDestLongitude");
}

function Exif_getGPSDestBearing()
{
	var value = Exif_resolveFraction(this.getValue("IFDGPS/GPSDestBearing"), 3);
	if (value == null)
		return null;
	return value + " " + Exif_GPSNorthType(this.getValue("IFDGPS/GPSDestBearingRef"));
}

function Exif_getGPSDestDistance()
{
	var value = Exif_resolveFraction(this.getValue("IFDGPS/GPSDestDistance"), 3);
	if (value == null)
		return null;
	var units = this.getValue("IFDGPS/GPSDestDistanceRef");
	if (units == 'K')
		units = "Kilometer";
	else if (units == 'M')
		units = "Mile";
	else if (units == 'N')
		units = " Nautical Mile"
	else
		units = "";
	if (units  &&  value != 1)
			units += "s";
	return value + " " + units;
}

function Exif_getGPSDifferential()
{
	return Exif_Lookup(this.getValue("IFDGPS/GPSDifferential"), Exif_GPSCorrection);
}

function Exif_getLens()
{
	return this.getValue("IFDMakerNote/Lens");
}

function Exif_getLensType()
{
	return this.getValue("IFDMakerNote/LensType");
}

function Exif_getFocusMode()
{
	return this.getValue("IFDMakerNote/FocusMode");
}

function Exif_getAFAreaMode()
{
	return this.getValue("IFDMakerNote/AFAreaMode");
}

function Exif_getVibrationReduction()
{
	return this.getValue("IFDMakerNote/VibrationReduction");
}

function Exif_getFlashMode()
{
	return this.getValue("IFDMakerNote/FlashMode");
}

function Exif_getActiveDLighting()
{
	return this.getValue("IFDMakerNote/ActiveDLighting");
}

function Exif_getPictureControlName()
{
	return this.getValue("IFDMakerNote/PictureControlName");
}

function Exif_getPictureControlBase()
{
	return this.getValue("IFDMakerNote/PictureControlBase");
}

function Exif_getPCQuickAdjust()
{
	return this.getValue("IFDMakerNote/PictureControlQuickAdjust");
}

function Exif_getHueAdjustment()
{
	return this.getValue("IFDMakerNote/HueAdjustment");
}

function Exif_getFilterEffect()
{
	return this.getValue("IFDMakerNote/FilterEffect");
}

function Exif_getToningEffect()
{
	return this.getValue("IFDMakerNote/ToningEffect");
}

function Exif_getOpticalZoomMode()
{
	return this.getValue("IFDMakerNote/OpticalZoomMode");
}

function Exif_getBrightness()
{
	return this.getValue("IFDMakerNote/Brightness");
}

/*function Exif_getAFFineTune()
{
}

function Exif_getFlashDevice()
{
}

function Exif_getFlashSyncMode()
{
}

function Exif_getLongExposureMR()
{
}

function Exif_getImageAuthentication()
{
}*/


ExifMetadata.prototype                          = new Metadata();
ExifMetadata.prototype.constructor              = ExifMetadata;
ExifMetadata.prototype.loadFromFile             = ExifMetadata_loadFromFile;
ExifMetadata.prototype.loadFromMedia            = ExifMetadata_loadFromMedia;
ExifMetadata.prototype.blankDocument            = ExifMetadata_blank;

// read and write tags -- IFD0
ExifMetadata.prototype.getImageDescription      = Exif_getImageDescription;
ExifMetadata.prototype.setImageDescription      = Exif_setImageDescription;
ExifMetadata.prototype.getMake                  = Exif_getMake;
ExifMetadata.prototype.setMake                  = Exif_setMake;
ExifMetadata.prototype.getModel                 = Exif_getModel;
ExifMetadata.prototype.setModel                 = Exif_setModel;
ExifMetadata.prototype.getOrientation           = Exif_getOrientation;
ExifMetadata.prototype.setOrientation           = Exif_setOrientation;
ExifMetadata.prototype.getSoftware              = Exif_getSoftware;
ExifMetadata.prototype.setSoftware              = Exif_setSoftware;
ExifMetadata.prototype.getArtist                = Exif_getArtist;
ExifMetadata.prototype.setArtist                = Exif_setArtist;
ExifMetadata.prototype.getDateTime              = Exif_getDateTime;
ExifMetadata.prototype.setDateTime              = Exif_setDateTime;
ExifMetadata.prototype.getPhotographerCopyright = Exif_getPhotographerCopyright;
ExifMetadata.prototype.setPhotographerCopyright = Exif_setPhotographerCopyright;
ExifMetadata.prototype.getEditorCopyright       = Exif_getEditorCopyright;
ExifMetadata.prototype.setEditorCopyright       = Exif_setEditorCopyright;
//  read only
ExifMetadata.prototype.getImageWidth            = Exif_getImageWidth;
ExifMetadata.prototype.getImageLength           = Exif_getImageLength;
ExifMetadata.prototype.getBitsPerSample         = Exif_getBitsPerSample;
ExifMetadata.prototype.getCompression           = Exif_getCompression;
ExifMetadata.prototype.getPhotometricInterpretation = Exif_getPhotometricInterpretation;
ExifMetadata.prototype.getPlanarConfiguration   = Exif_getPlanarConfiguration;
ExifMetadata.prototype.getYCbCrSubSampling      = Exif_getYCbCrSubSampling;
ExifMetadata.prototype.getYCbCrPositioning      = Exif_getYCbCrPositioning;
ExifMetadata.prototype.getXResolution           = Exif_getXResolution;
ExifMetadata.prototype.getYResolution           = Exif_getYResolution;
ExifMetadata.prototype.getResolutionUnit        = Exif_getResolutionUnit;
ExifMetadata.prototype.getWhitePoint            = Exif_getWhitePoint;
ExifMetadata.prototype.getPrimaryChromaticities = Exif_getPrimaryChromaticities;
ExifMetadata.prototype.getYCbCrCoefficients     = Exif_getYCbCrCoefficients;
ExifMetadata.prototype.getReferenceBlackWhite   = Exif_getReferenceBlackWhite;
//  IFDExif
ExifMetadata.prototype.getVersion               = Exif_getVersion;
ExifMetadata.prototype.setVersion               = Exif_setVersion;
ExifMetadata.prototype.getFlashPixVersion       = Exif_getFlashPixVersion;
ExifMetadata.prototype.setFlashPixVersion       = Exif_setFlashPixVersion;
ExifMetadata.prototype.getColorspace            = Exif_getColorspace;
ExifMetadata.prototype.getPixelXDimension       = Exif_getPixelXDimension;
ExifMetadata.prototype.getPixelYDimension       = Exif_getPixelYDimension;
ExifMetadata.prototype.getComponentsConfiguration = Exif_getComponentsConfiguration;
ExifMetadata.prototype.getCompressedBitsPerPixel = Exif_getCompressedBitsPerPixel;
ExifMetadata.prototype.getUserComment = Exif_getUserComment;
ExifMetadata.prototype.setUserComment = Exif_setUserComment;
ExifMetadata.prototype.getRelatedSoundFile = Exif_getRelatedSoundFile;
ExifMetadata.prototype.getDateTimeOriginal = Exif_getDateTimeOriginal;
ExifMetadata.prototype.getDateTimeDigitized = Exif_getDateTimeDigitized;
ExifMetadata.prototype.getSubSecTime = Exif_getSubSecTime;
ExifMetadata.prototype.getSubSecTimeOriginal = Exif_getSubSecTimeOriginal;
ExifMetadata.prototype.getSubSecTimeDigitized = Exif_getSubSecTimeDigitized;
ExifMetadata.prototype.getExposureTime = Exif_getExposureTime;
ExifMetadata.prototype.getShutterSpeedValue = Exif_getShutterSpeedValue;
ExifMetadata.prototype.getApertureValue = Exif_getApertureValue;
ExifMetadata.prototype.getBrightnessValue = Exif_getBrightnessValue;
ExifMetadata.prototype.getExposureBiasValue = Exif_getExposureBiasValue;
ExifMetadata.prototype.getMaxApertureValue = Exif_getMaxApertureValue;
ExifMetadata.prototype.getSubjectDistance = Exif_getSubjectDistance;
ExifMetadata.prototype.getMeteringMode = Exif_getMeteringMode;
ExifMetadata.prototype.getLightSource = Exif_getLightSource;
ExifMetadata.prototype.getFlash = Exif_getFlash;
ExifMetadata.prototype.getFocalLength = Exif_getFocalLength;
ExifMetadata.prototype.getFNumber = Exif_getFNumber;
ExifMetadata.prototype.getExposureProgram = Exif_getExposureProgram;
ExifMetadata.prototype.getSpectralSensitivity = Exif_getSpectralSensitivity;
ExifMetadata.prototype.getISOSpeedRatings = Exif_getISOSpeedRatings;
ExifMetadata.prototype.getOECF = Exif_getOECF;
ExifMetadata.prototype.getFlashEnergy = Exif_getFlashEnergy;
ExifMetadata.prototype.getSpatialFrequencyResponse = Exif_getSpatialFrequencyResponse;
ExifMetadata.prototype.getFocalPlaneXResolution = Exif_getFocalPlaneXResolution;
ExifMetadata.prototype.getFocalPlaneYResolution = Exif_getFocalPlaneYResolution;
ExifMetadata.prototype.getFocalPlaneResolutionUnit = Exif_getFocalPlaneResolutionUnit;
ExifMetadata.prototype.getSubjectLocation = Exif_getSubjectLocation;
ExifMetadata.prototype.getExposureIndex = Exif_getExposureIndex;
ExifMetadata.prototype.getSensingMethod = Exif_getSensingMethod;
ExifMetadata.prototype.getFileSource = Exif_getFileSource;
ExifMetadata.prototype.getSceneType = Exif_getSceneType;
ExifMetadata.prototype.getCFAPattern = Exif_getCFAPattern;
// Exif 2.2
ExifMetadata.prototype.getSubjectArea = Exif_getSubjectArea;
ExifMetadata.prototype.getMakerNote = Exif_getMakerNote;
ExifMetadata.prototype.getCustomRendered = Exif_getCustomRendered;
ExifMetadata.prototype.getExposureMode = Exif_getExposureMode;
ExifMetadata.prototype.getWhiteBalance = Exif_getWhiteBalance;
ExifMetadata.prototype.getDigitalZoomRatio = Exif_getDigitalZoomRatio;
ExifMetadata.prototype.getFocalLengthIn35mmFilm = Exif_getFocalLengthIn35mmFilm;
ExifMetadata.prototype.getSceneCaptureType = Exif_getSceneCaptureType;
ExifMetadata.prototype.getGainControl = Exif_getGainControl;
ExifMetadata.prototype.getContrast = Exif_getContrast;
ExifMetadata.prototype.getSaturation = Exif_getSaturation;
ExifMetadata.prototype.getSharpness = Exif_getSharpness;
ExifMetadata.prototype.getDeviceSettingDescription = Exif_getDeviceSettingDescription;
ExifMetadata.prototype.getSubjectDistanceRange = Exif_getSubjectDistanceRange;
ExifMetadata.prototype.getImageUniqueID = Exif_getImageUniqueID;
//IFDGPS
ExifMetadata.prototype.getGPSVersionID = Exif_getGPSVersionID;
ExifMetadata.prototype.getGPSLatitude = Exif_getGPSLatitude;
ExifMetadata.prototype.getGPSLongitude = Exif_getGPSLongitude;
ExifMetadata.prototype.getGPSAltitude = Exif_getGPSAltitude;
ExifMetadata.prototype.getGPSTimeStamp = Exif_getGPSTimeStamp;
ExifMetadata.prototype.getGPSDateStamp = Exif_getGPSDateStamp;
ExifMetadata.prototype.getGPSFullUTC = Exif_getGPSFullUTC;
ExifMetadata.prototype.getGPSSatellites = Exif_getGPSSatellites;
ExifMetadata.prototype.getGPSStatus = Exif_getGPSStatus;
ExifMetadata.prototype.getGPSMeasureMode = Exif_getGPSMeasureMode;
ExifMetadata.prototype.getGPSDOP = Exif_getGPSDOP;
ExifMetadata.prototype.getGPSSpeed = Exif_getGPSSpeed;
ExifMetadata.prototype.getGPSTrack = Exif_getGPSTrack;
ExifMetadata.prototype.getGPSImgDirection = Exif_getGPSImgDirection;
ExifMetadata.prototype.getGPSMapDatum = Exif_getGPSMapDatum;
ExifMetadata.prototype.getGPSDestLatitude = Exif_getGPSDestLatitude;
ExifMetadata.prototype.getGPSDestLongitude = Exif_getGPSDestLongitude;
ExifMetadata.prototype.getGPSDestBearing = Exif_getGPSDestBearing;
ExifMetadata.prototype.getGPSDestDistance = Exif_getGPSDestDistance;
ExifMetadata.prototype.getGPSDifferential = Exif_getGPSDifferential;
//IFDMakerNote
ExifMetadata.prototype.getLens						= Exif_getLens;
ExifMetadata.prototype.getLensType					= Exif_getLensType;
ExifMetadata.prototype.getFocusMode					= Exif_getFocusMode;
ExifMetadata.prototype.getAFAreaMode				= Exif_getAFAreaMode;
ExifMetadata.prototype.getVibrationReduction		= Exif_getVibrationReduction;
ExifMetadata.prototype.getFlashMode					= Exif_getFlashMode;
ExifMetadata.prototype.getActiveDLighting			= Exif_getActiveDLighting;
ExifMetadata.prototype.getPictureControlName		= Exif_getPictureControlName;
ExifMetadata.prototype.getPictureControlBase		= Exif_getPictureControlBase;
ExifMetadata.prototype.getPictureControlQuickAdjust	= Exif_getPCQuickAdjust;
ExifMetadata.prototype.getHueAdjustment				= Exif_getHueAdjustment;
ExifMetadata.prototype.getFilterEffect				= Exif_getFilterEffect;
ExifMetadata.prototype.getToningEffect				= Exif_getToningEffect;
ExifMetadata.prototype.getOpticalZoomMode			= Exif_getOpticalZoomMode
ExifMetadata.prototype.getBrightness				= Exif_getBrightness
//ExifMetadata.prototype.getAFFineTune				= Exif_getAFFineTune;
//ExifMetadata.prototype.getFlashDevice				= Exif_getFlashDevice;
//ExifMetadata.prototype.getFlashSyncMode			= Exif_getFlashSyncMode;
//ExifMetadata.prototype.getLongExposureMR			= Exif_getLongExposureMR;
//ExifMetadata.prototype.getImageAuthentication		= Exif_getImageAuthentication;
