/*+
 * IPTCMetadata.ms
 *
 *  Contains the Metadata subclass for Exif.
 *
 *  See method descriptions below.
 *
-*/

#include "sys:/Metadata.ms"

/* ----------------------------------------------------------------------------
 *  IPTCMetadata
 *
 *  construct a new Metadata object from the file.
 *
 *  fileName    - (string) the name of the file to get the data from
 *  validate    - (boolean) true to validate (default)
 *
 *  example:
 *      // load metadata from image.
 *      metadata = new Metadata("image.jpg", "IPTC", true);
 -----------------------------------------------------------------------------*/
function IPTCMetadata(validate)
{
    this._init(validate);
    this.blankDocument();
}

/* ----------------------------------------------------------------------------
 *  IPTCMetadata_blank
 *
 *  create blank IPTC document.
 *
 *
 *  example:
 *      // load metadata from image.
 *      metadata = new IPTCMetadata();
 *      metadata.blankDocument();
 -----------------------------------------------------------------------------*/
function IPTCMetadata_blank
{
    this._clear();
    var dom = this.getDOM();

    var root = dom.createElement("IPTC");
    root.setAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
    var xsdDocument = getPropertyValue("paths.SysPath") + "/IPTC.xsd";
    root.setAttribute("xsi:noNamespaceSchemaLocation", xsdDocument);
    dom.appendChild(root);

    var vers = dom.createElement("Version");
    vers.text = "2";
    root.appendChild(vers);
    if (this.doValidation)
        this.validate();
}

/* ----------------------------------------------------------------------------
 *  IPTCMetadata_loadFromFile
 *
 *  load IPTCMetadata object from the file.
 *
 *  fileName    - (string) the name of the file to get the data from
 *
 *  example:
 *      // load metadata from image.
 *      metadata = new IPTCMetadata();
 *      metadata.loadFromFile("image.jpg");
 -----------------------------------------------------------------------------*/
function IPTCMetadata_loadFromFile(fileName)
{
    this._loadFromFile(fileName, "IPTC");
}

/* ----------------------------------------------------------------------------
 *  IPTCMetadata_loadFromFile
 *
 *  load IPTCMetadata object from the media.
 *
 *  media       - (string) the name of the file to get the data from
 *
 *  example:
 *      // load metadata from image.
 *      var m = new Media();
 *      m.load(name @ "image.jpg");
 *      metadata = new IPTCMetadata();
 *      metadata.loadFromMedia(m);
 -----------------------------------------------------------------------------*/
function IPTCMetadata_loadFromMedia(media)
{
    this._loadFromMedia(media, "IPTC");
}

IPTC_ElementLength["ObjectTypeReference"] = 67;
IPTC_ElementLength["ObjectAttributeReference"] = 68;
IPTC_ElementLength["ObjectName"] = 64;
IPTC_ElementLength["EditStatus"] = 64;
IPTC_ElementLength["EditorialUpdate"] = 2;
IPTC_ElementLength["SubjectReference"] = 236;
IPTC_ElementLength["Category"] = 3;
IPTC_ElementLength["SupplementalCategory"] = 32;
IPTC_ElementLength["FixtureIdentifier"] = 32;
IPTC_ElementLength["Keywords"] = 64;
IPTC_ElementLength["ObjectCycle"] = 1;

function IPTC_CheckLength(key, s)
{
    if (s != null)
    {
        var maxLen = IPTC_ElementLength[key];
        if (maxLen != null)
        {
            if (s.length > maxLen)
                return s.substr(0, maxLen);
        }
    }
    return s;
}

function IPTC_ConvertDate(datePart, timePart)
{
    if (datePart == null)
        return datePart;

    var year = parseInt(datePart.substr(0,4), 10);
    var month = parseInt(datePart.substr(4,2), 10) - 1;
    var day = parseInt(datePart.substr(6,2), 10);

    if (timePart != null)
    {
        var hour = parseInt(timePart.substr(0,2), 10);
        var min = parseInt(timePart.substr(2,2), 10);
        var sec = parseInt(timePart.substr(4,2), 10);
        var pos = timePart.substr(6,1) == "+";
        var hrskew = parseInt(timePart.substr(7,2), 10);
        var minskew = parseInt(timePart.substr(9,2), 10);
        if (pos)
        {
            hour += hrskew;
            min += minskew;
        }
        else
        {
            hour -= hrskew;
            min -= minskew;
        }
    }
    else
    {
        hour = min = sec = skew = 0;
    }

    return new Date(Date.UTC(year, month, day, hour, min, sec, 0));
}

function IPTC_Pad(val, digs)
{
    var s = val.toString();
    while(s.length < digs)
        s = '0' + s;

    return s;
}

function IPTC_SetDate(date)
{
    var year = IPTC_Pad(date.getFullYear(), 4);
    var mon = IPTC_Pad(date.getMonth() + 1, 2);
    var day = IPTC_Pad(date.getDate(), 2)
    var hour = IPTC_Pad(date.getHours(), 2)
    var min = IPTC_Pad(date.getMinutes(), 2)
    var sec = IPTC_Pad(date.getSeconds(), 2)
    var tz = date.getTimezoneOffset();
    if (tz < 0)
        plus = "-";
    else
        plus = "+";
    tz = Math.abs(tz);
    var tzhrs = tz / 60;
    var tzmins = tz - tzhrs * 60;

    return [ year + mon + day, hour + min + sec + plus + IPTC_Pad(tzhrs, 2) + IPTC_Pad(tzmins, 2) ];
}

function IPTC_checkValue(value)
{
    if (value == null)
        error("Cannot set iptc value with null or undefined value.");
}

function IPTC_emptyValue(value)
{
    IPTC_checkValue(value);
    return (typeof(value) == "string" && value.length == 0);
}

function IPTC_checkArray(arr)
{
    IPTC_checkValue(arr);
    if (typeof(arr) != "object" || arr.length == null)
        error("Value must be an array.");
}

function IPTC_setDateTimeValue(dateTag, timeTag, value)
{
    if (IPTC_emptyValue(value))
    {
        this.setValue(dateTag, value);
        this.setValue(timeTag, value);
    }
    else if (typeof(value) == "string")
    {
        this.setValue(dateTag, value);
    }
    else
    {
        var arr = IPTC_SetDate(value);
        this.setValue(dateTag, arr[0]);
        this.setValue(timeTag, arr[1]);
    }
    if (this.doValidation)
        this.validate();
}

function IPTC_getVersion()
{
    return this.getValue("Version");
}

function IPTC_setVersion(value)
{
    if (IPTC_emptyValue(value))
        error("Version field may not be removed.");
    var dValue = parseFloat(value);
    if (isNaN(dValue) || dValue.toString() != value.toString())
        error("IPTC version must be a number.");
    this.setValue("Version", value.toString());
}

//  return string
function IPTC_getObjectTypeReference()
{
    return IPTC_CheckLength("ObjectTypeReference", this.getValue("ObjectTypeReference"));
}

//  value is string
function IPTC_setObjectTypeReference(value)
{
    IPTC_checkValue(value);
    value = IPTC_CheckLength("ObjectTypeReference", value);
    this.setValue("ObjectTypeReference", value);
    if (this.doValidation)
        this.validate();
}

//  return array
function IPTC_getObjectAttributeReference()
{
    var arr = this.getValue("ObjectAttributeReference");
    if (arr != null)
        for (var elem in arr)
            arr[elem] = IPTC_CheckLength("ObjectAttributeReference", arr[elem]);
    return arr;
}

//  value is array
function IPTC_setObjectAttributeReferenceArray(arr)
{
    IPTC_checkArray(arr)
    for (var elem in arr)
        arr[elem] = IPTC_CheckLength("ObjectAttributeReference", arr[elem]);

    this.setValue("ObjectAttributeReference", arr);
    if (this.doValidation)
        this.validate();
}

//  arguements are string
//  iptc.setAttributeReference(value1, value2,...)
function IPTC_setObjectAttributeReference()
{
    if (arguments.length == 0)
    {
        error("Expected at least one value in setObjectAttributeReference.");
    }
    else
    {
        var arr = new Array();
        for (var i = 0; i < arguments.length; i++)
            arr.push(arguments[i]);
        this.setObjectAttributeReferenceArray(arr);
    }
}

//  value is string
function IPTC_addObjectAttributeReference(value)
{
    if (IPTC_emptyValue(value))
        return;
    arr = this.getValue("ObjectAttributeReference");
    if (arr == null)
        arr = new Array();
    arr.push(value);
    this.setObjectAttributeReferenceArray(arr);
}


//  return string
function IPTC_getObjectName(value)
{
    return IPTC_CheckLength("ObjectName", this.getValue("ObjectName"));
}

function IPTC_setObjectName(value)
{
    IPTC_checkValue(value);
    value = IPTC_CheckLength("ObjectName", value);
    this.setValue("ObjectName", value);
    if (this.doValidation)
        this.validate();
}

//  return string
function IPTC_getEditStatus()
{
    return IPTC_CheckLength("EditStatus", this.getValue("EditStatus"));
}

function IPTC_setEditStatus(value)
{
    IPTC_checkValue(value);
    value = IPTC_CheckLength("EditStatus", value);
    this.setValue("EditStatus", value);
    if (this.doValidation)
        this.validate();
}

//  return string
function IPTC_getEditorialUpdate()
{
    return IPTC_CheckLength("EditorialUpdate", this.getValue("EditorialUpdate"));
}

function IPTC_setEditorialUpdate(value)
{
    IPTC_checkValue(value);
    value = IPTC_CheckLength("EditorialUpdate", value);
    this.setValue("EditorialUpdate", value);
    if (this.doValidation)
        this.validate();
}

//  return number
function IPTC_getUrgency()
{
    var urg = this.getValue("Urgency");
    if (urg == null)
        return urg;
    return parseInt(urg);
}

function IPTC_setUrgency(value)
{
    if (!IPTC_emptyValue(value))
    {
        var iVal = parseInt(value);
        if (isNaN(iVal) || iVal < 0 || iVal > 9)
            error("Urgency must be an integer between 0 and 9");
    }
    this.setValue("Urgency", value);
    if (this.doValidation)
        this.validate();
}

//  returns an array
function IPTC_getSubjectReference()
{
    var arr = this.getValue("SubjectReference");
    if (arr != null)
        for (var elem in arr)
            arr[elem] = IPTC_CheckLength("SubjectReference", arr[elem]);
    return arr;
}

function IPTC_setSubjectReferenceArray(arr)
{
    IPTC_checkArray(arr)
    for (var elem in arr)
        arr[elem] = IPTC_CheckLength("SubjectReference", arr[elem]);
    this.setValue("SubjectReference", arr);
    if (this.doValidation)
        this.validate();
}

function IPTC_setSubjectReference()
{
    if (arguments.length == 0)
    {
        error("Expected at least one parameter for setSubjectReference.");
    }
    else
    {
        var arr = new Array();
        for (var i = 0; i < arguments.length; i++)
            arr.push(arguments[i]);
        this.setSubjectReferenceArray(arr);
    }
}

function IPTC_addSubjectReference(value)
{
    if (IPTC_emptyValue(value))
        return;
    var arr = this.getValue("SubjectReference");
    if (arr == null)
        arr = new Array();

    arr.push(value);
    this.setSubjectReferenceArray(arr);
}

//  return string
function IPTC_getCategory()
{
    return IPTC_CheckLength("Category", this.getValue("Category"));
}

function IPTC_setCategory(value)
{
    IPTC_checkValue(value);
    value = IPTC_CheckLength("Category", value);
    this.setValue("Category", value);
    if (this.doValidation)
        this.validate();
}

//  return array
function IPTC_getSupplementalCategory()
{
    var arr = this.getValue("SupplementalCategory");
    if (arr != null)
        for (var elem in arr)
            arr[elem] = IPTC_CheckLength("SupplementalCategory", arr[elem]);
    return arr;
}

function IPTC_addSupplementalCategory(value)
{
    if (IPTC_emptyValue(value))
        return;
    var arr = this.getValue("SupplementalCategory");
    if (arr == null)
        arr = new Array();

    arr.push(value);
    this.setSupplementalCategoryArray(arr);
}

function IPTC_setSupplementalCategoryArray(arr)
{
    IPTC_checkArray(arr);
    for (var elem in arr)
        arr[elem] = IPTC_CheckLength("SupplementalCategory", arr[elem]);
    this.setValue("SupplementalCategory", arr);
    if (this.doValidation)
        this.validate();
}

function IPTC_setSupplementalCategory()
{
    if (arguments.length == 0)
    {
        error("Expected at least one parameter for setSupplementalCategory.");
    }
    else
    {
        var arr = new Array();
        for (var i = 0; i < arguments.length; i++)
            arr.push(arguments[i]);
        this.setSupplementalCategoryArray(arr);
    }
}

//  return string
function IPTC_getFixtureIdentifier()
{
    return IPTC_CheckLength("FixtureIdentifier", this.getValue("FixtureIdentifier"));
}

function IPTC_setFixtureIdentifier(value)
{
    IPTC_checkValue(value);
    value = IPTC_CheckLength("FixtureIdentifier", value);
    this.setValue("FixtureIdentifier", value);
    if (this.doValidation)
        this.validate();
}

//  return array
function IPTC_getKeywords()
{
    var arr = this.getValue("Keywords");
    if (arr != null)
        for (var elem in arr)
            arr[elem] = IPTC_CheckLength("Keywords", arr[elem]);
    return arr;
}

function IPTC_addKeywords()
{
    var arr = this.getValue("Keywords");
    if (arr == null)
        arr = new Array();
    for (var i = 0; i < arguments.length; i++)
    {
        if (arguments[i].length > 0)
            arr.push(arguments[i]);
    }

    this.setKeywordsArray(arr);
}

function IPTC_setKeywordsArray(arr)
{
    IPTC_checkArray(arr);
    for (var elem in arr)
        arr[elem] = IPTC_CheckLength("Keywords", arr[elem]);
    this.setValue("Keywords", arr);
    if (this.doValidation)
        this.validate();
}

function IPTC_setKeywords()
{
    if (arguments.length == 0)
    {
        error("Expected at least one parameter for setKeywords.");
    }
    else
    {
        var arr = new Array();
        for (var i = 0; i < arguments.length; i++)
            arr.push(arguments[i]);
        this.setKeywordsArray(arr);
    }
}

// array of objects
function IPTC_getContentLocation()
{
    return this.getValue("ContentLocation");
}

// string
function IPTC_getContentLocationCode(which)
{
    if (which == null)
        which = 0;
    var arr = this.getValue("ContentLocation");
    if (arr != null)
        return arr[which].ContentLocationCode;
    return null;
}

// array of objects
function IPTC_getContentLocationName(which)
{
    if (which == null)
        which = 0;
    var arr = this.getValue("ContentLocation");
    if (arr != null)
        return arr[which].ContentLocationName;
    return null;
}

function IPTC_addContentLocation(code, name)
{
    IPTC_checkValue(code);

    var currValue = new Object;
    currValue["ContentLocationCode"] = code;
    if (name != null)
        currValue["ContentLocationName"] = name;
    var arr = this.getValue("ContentLocation");
    if (arr == null)
        arr = new Array();

    arr.push(currValue)
    this.setValue("ContentLocation", arr);
    if (this.doValidation)
        this.validate();
}

function IPTC_setContentLocation(code, name)
{
    if (IPTC_emptyValue(code))
    {
        this.setValue("ContentLocation", "");
    }
    else
    {
        var currValue = new Object;
        currValue["ContentLocationCode"] = code;
        if (name != null)
            currValue["ContentLocationName"] = name;

        var arr = new Array();
        arr.push(currValue)
        this.setValue("ContentLocation", arr);
    }
    if (this.doValidation)
        this.validate();
}

//  return Date
function IPTC_getReleaseDate()
{
    return IPTC_ConvertDate(this.getValue("ReleaseDate"),
            this.getValue("ReleaseTime"));
}

function IPTC_setReleaseDate(value)
{
    this._setDateTimeValue("ReleaseDate", "ReleaseTime", value);
}

function IPTC_setReleaseTime(value)
{
    IPTC_checkValue(value);
    if (typeof(value) == "string")
        this.setValue("ReleaseTime", value);
    else
        error("Use setReleaseDate to set the date from a Date object");
    if (this.doValidation)
        this.validate();
}

//  return Date
function IPTC_getExpirationDate()
{
    return IPTC_ConvertDate(this.getValue("ExpirationDate"),
            this.getValue("ExpirationTime"));
}

function IPTC_setExpirationDate(value)
{
    this._setDateTimeValue("ExpirationDate", "ExpirationTime", value);
}

function IPTC_setExpirationTime(value)
{
    IPTC_checkValue(value);
    if (typeof(value) == "string")
        this.setValue("ExpirationTime", value);
    else
        error("Use setExpirationDate to set the date from a Date object");
    if (this.doValidation)
        this.validate();
}

function IPTC_getSpecialInstructions()
{
    return this.getValue("SpecialInstructions");
}

function IPTC_setSpecialInstructions(value)
{
    IPTC_checkValue(value);
    this.setValue("SpecialInstructions", value);
    if (this.doValidation)
        this.validate();
}

function IPTC_getActionAdvised()
{
    return this.getValue("ActionAdvised");
}

function IPTC_setActionAdvised(value)
{
    IPTC_checkValue(value);
    this.setValue("ActionAdvised", value);
    if (this.doValidation)
        this.validate();
}

//  return array of objects
function IPTC_getReference()
{
    var ref = this.getValue("Reference");
    if (ref != null)
    {
        for (var i in ref)
        {
            var entry = ref[i];
            entry.ReferenceDate = IPTC_ConvertDate(entry.ReferenceDate);
        }
    }
    return ref;
}

function IPTC_getReferenceService(which)
{
    if (which == null)
        which = 0;

    var obj = this.getValue("Reference");
    if (obj)
        return obj[which].ReferenceService;
    return null;
}

function IPTC_getReferenceDate(which)
{
    if (which == null)
        which = 0;

    var obj = this.getValue("Reference");
    if (obj)
        return IPTC_ConvertDate(obj[which].ReferenceDate);
    return null;
}

function IPTC_getReferenceNumber(which)
{
    if (which == null)
        which = 0;

    var obj = this.getValue("Reference");
    if (obj)
        return obj[which].ReferenceNumber;
    return null;
}

function IPTC_addReference(service, date, number)
{
    var obj = new Object();
    obj.ReferenceService = service.toString();;
    obj.ReferenceDate = IPTC_SetDate(date)[0];
    obj.ReferenceNumber = number.toString();

    var arr = this.getValue("Reference");
    if (arr == null)
        arr = new Array();
    arr.push(obj);
    this.setValue("Reference", arr);
    if (this.doValidation)
        this.validate();
}

function IPTC_setReference(service, date, number)
{
    if (IPTC_emptyValue(service))
    {
        this.setValue("Reference", "");
    }
    else
    {
        var obj = new Object();
        obj.ReferenceService = service.toString();;
        obj.ReferenceDate = IPTC_SetDate(date)[0];
        obj.ReferenceNumber = number.toString();

        var arr = new Array();
        arr.push(obj);
        this.setValue("Reference", arr);
    }
    if (this.doValidation)
        this.validate();
}

function IPTC_getDateCreated()
{
    return IPTC_ConvertDate(this.getValue("DateCreated"),
            this.getValue("TimeCreated"));
}

function IPTC_setDateCreated(value)
{
    this._setDateTimeValue("DateCreated", "TimeCreated", value);
}

function IPTC_setTimeCreated(value)
{
    if (typeof(value) == "string")
        this.setValue("TimeCreated", value);
    else
        error("Use setDateCreated to set the date from a Date object");
    if (this.doValidation)
        this.validate();
}

function IPTC_getDigitalCreationDate()
{
    return IPTC_ConvertDate(this.getValue("DigitalCreationDate"),
            this.getValue("DigitalCreationTime"));
}

function IPTC_setDigitalCreationDate(value)
{
    this._setDateTimeValue("DigitalCreationDate", "DigitalCreationTime", value);
}

function IPTC_setDigitalCreationTime(value)
{
    if (typeof(value) == "string")
        this.setValue("DigitalCreationTime", value);
    else
        error("Use setDigitalCreationDate to set the date from a Date object");
    if (this.doValidation)
        this.validate();
}

function IPTC_getOriginatingProgram()
{
    return this.getValue("OriginatingProgram");
}

function IPTC_setOriginatingProgram(value)
{
    IPTC_checkValue(value);
    this.setValue("OriginatingProgram", value);
}

function IPTC_getProgramVersion()
{
    return this.getValue("ProgramVersion");
}

function IPTC_setProgramVersion(value)
{
    IPTC_checkValue(value);
    this.setValue("ProgramVersion", value);
    if (this.doValidation)
        this.validate();
}

//  return array
function IPTC_getObjectCycle()
{
    return IPTC_CheckLength("ObjectCycle", this.getValue("ObjectCycle"));
}

function IPTC_setObjectCycle(value)
{
    IPTC_checkValue(value);
    value = IPTC_CheckLength("ObjectCycle", value);
    this.setValue("ObjectCycle", value);
    if (this.doValidation)
        this.validate();
}

//  return array obj object
function IPTC_getByLine()
{
    return this.getValue("ByLine");
}

function IPTC_addByLine(writer, title)
{
    if (IPTC_emptyValue(writer))
        return;
    var byline = new Object();
    byline.ByLineWriter = writer;
    if (title != null)
        byline.ByLineTitle = title;

    var arr = this.getValue("ByLine");
    if (arr == null)
        arr = new Array();

    arr.push(byline);
    this.setValue("ByLine", arr);
    if (this.doValidation)
        this.validate();
}

function IPTC_setByLineWriter(writer)
{
	this.addByLine(writer, null);
}

function IPTC_setByLineTitle(title)
{
	var writer = this.getByLineWriter();
	if (writer == null)
		writer = "";
	
	this.addByLine(writer, title);
}

function IPTC_setByLine(writer, title)
{
    if (IPTC_emptyValue(writer))
    {
        this.setValue("ByLine", "");
    }
    else
    {
        var byline = new Object();
        byline.ByLineWriter = writer;
        if (title != null)
            byline.ByLineTitle = title;

        var arr = new Array();
        arr.push(byline);
        this.setValue("ByLine", arr);
    }
    if (this.doValidation)
        this.validate();
}

function IPTC_getByLineWriter(which)
{
    if (which == null)
    {
        which = 0;
	}
	else if (typeof which != "number")
	{
		which = parseInt(which);
		if (isNaN(which))
			which = 0;
	}
	
    byline = this.getValue("ByLine");
    if (byline)
        return byline[which].ByLineWriter;
    return null;
}

function IPTC_getByLineTitle(which)
{
    if (which == null)
    {
        which = 0;
	}
	else if (typeof which != "number")
	{
		which = parseInt(which);
		if (isNaN(which))
			which = 0;
	}
	
    byline = this.getValue("ByLine");
    if (byline)
        return byline[which].ByLineTitle;
    return null;
}

function IPTC_getCity()
{
    return this.getValue("City");
}

function IPTC_setCity(value)
{
    IPTC_checkValue(value);
    this.setValue("City", value);
    if (this.doValidation)
        this.validate();
}

function IPTC_getSublocation()
{
    return this.getValue("Sublocation");
}

function IPTC_setSublocation(value)
{
    IPTC_checkValue(value);
    this.setValue("Sublocation", value);
    if (this.doValidation)
        this.validate();
}

function IPTC_getState()
{
    return this.getValue("State");
}

function IPTC_setState(value)
{
    IPTC_checkValue(value);
    this.setValue("State", value);
    if (this.doValidation)
        this.validate();
}

function IPTC_getCountryCode()
{
    return this.getValue("CountryCode");
}

function IPTC_setCountryCode(value)
{
    IPTC_checkValue(value);
    this.setValue("CountryCode", value);
    if (this.doValidation)
        this.validate();
}

function IPTC_getCountryName()
{
    return this.getValue("CountryName");
}

function IPTC_setCountryName(value)
{
    IPTC_checkValue(value);
    this.setValue("CountryName", value);
    if (this.doValidation)
        this.validate();
}

function IPTC_getOriginalTransmissionReference()
{
    return this.getValue("OriginalTransmissionReference");
}

function IPTC_setOriginalTransmissionReference(value)
{
    IPTC_checkValue(value);
    this.setValue("OriginalTransmissionReference", value);
    if (this.doValidation)
        this.validate();
}

function IPTC_getHeadline()
{
    return this.getValue("Headline");
}

function IPTC_setHeadline(value)
{
    IPTC_checkValue(value);
    this.setValue("Headline", value);
    if (this.doValidation)
        this.validate();
}

function IPTC_getCredit()
{
    return this.getValue("Credit");
}

function IPTC_setCredit(value)
{
    IPTC_checkValue(value);
    this.setValue("Credit", value);
    if (this.doValidation)
        this.validate();
}

function IPTC_getSource()
{
    return this.getValue("Source");
}

function IPTC_setSource(value)
{
    IPTC_checkValue(value);
    this.setValue("Source", value);
    if (this.doValidation)
        this.validate();
}

function IPTC_getCopyrightNotice()
{
    return this.getValue("CopyrightNotice");
}

function IPTC_setCopyrightNotice(value)
{
    IPTC_checkValue(value);
    this.setValue("CopyrightNotice", value);
    if (this.doValidation)
        this.validate();
}

//  returns array
function IPTC_getContact()
{
    return this.getValue("Contact");
}

function IPTC_addContact(value)
{
    if (IPTC_emptyValue(value))
        return;
    var arr = this.getValue("Contact");
    if (arr == null)
        arr = new Array();
    arr.push(value);
    this.setValue("Contact", arr);
    if (this.doValidation)
        this.validate();
}

function IPTC_setContactArray(arr)
{
    IPTC_checkArray(arr);
    this.setValue("Contact", arr);
    if (this.doValidation)
        this.validate();
}

function IPTC_setContact()
{
    if (arguments.length == 0)
        error("Expected at lease one parameter for setContact.");
    else
    {
        var arr = new Array();
        for (var i = 0; i < arguments.length; i++)
            arr.push(arguments[i]);
        this.setContactArray(arr);
    }
}

function IPTC_getCaption()
{
    return this.getValue("Caption");
}

function IPTC_setCaption(value)
{
    IPTC_checkValue(value);
    this.setValue("Caption", value);
    if (this.doValidation)
        this.validate();
}

function IPTC_getWriter()
{
    return this.getValue("Writer");
}

function IPTC_addWriter(value)
{
    if (IPTC_emptyValue(value))
        return;
    var arr = this.getValue("Writer");
    if (arr == null)
        arr = new Array();
    arr.push(value);
    this.setWriterArray(arr);
}

function IPTC_setWriterArray(arr)
{
    IPTC_checkArray(arr);
    this.setValue("Writer", arr);
    if (this.doValidation)
        this.validate();
}

function IPTC_setWriter()
{
    if (arguments.length == 0)
        error("Expected at least one parameter for setWriter.");
    else
    {
        var arr = new Array();
        for (var i = 0; i < arguments.length; i++)
            arr.push(arguments[i]);
        this.setWriterArray(arr);
    }
}

function IPTC_getImageType()
{
    return this.getValue("ImageType");
}

function IPTC_setImageType(value)
{
    IPTC_checkValue(value);
    this.setValue("ImageType", value);
    if (this.doValidation)
        this.validate();
}

function IPTC_getImageOrientation()
{
    return this.getValue("ImageOrientation");
}

function IPTC_setImageOrientation(value)
{
    IPTC_checkValue(value);
    this.setValue("ImageOrientation", value);
    if (this.doValidation)
        this.validate();
}

function IPTC_getLanguageIdentifier()
{
    return this.getValue("LanguageIdentifier");
}

function IPTC_setLanguageIdentifier(value)
{
    IPTC_checkValue(value);
    this.setValue("LanguageIdentifier", value);
    if (this.doValidation)
        this.validate();
}

function IPTC_getAudioType()
{
    return this.getValue("AudioType");
}

function IPTC_setAudioType(value)
{
    IPTC_checkValue(value);
    this.setValue("AudioType", value);
    if (this.doValidation)
        this.validate();
}

function IPTC_getAudioSamplingRate()
{
    return this.getValue("AudioSamplingRate");
}

function IPTC_setAudioSamplingRate(value)
{
    IPTC_checkValue(value);
    this.setValue("AudioSamplingRate", value);
    if (this.doValidation)
        this.validate();
}

function IPTC_getAudioSamplingResolution()
{
    return this.getValue("AudioSamplingResolution");
}

function IPTC_setAudioSamplingResolution(value)
{
    IPTC_checkValue(value);
    this.setValue("AudioSamplingResolution", value);
    if (this.doValidation)
        this.validate();
}

function IPTC_getAudioDuration()
{
    return this.getValue("AudioDuration");
}

function IPTC_setAudioDuration(value)
{
    IPTC_checkValue(value);
    this.setValue("AudioDuration", value);
    if (this.doValidation)
        this.validate();
}

function IPTC_getAudioOutcue()
{
    return this.getValue("AudioOutcue");
}

function IPTC_setAudioOutcue(value)
{
    IPTC_checkValue(value);
    this.setValue("AudioOutcue", value);
    if (this.doValidation)
        this.validate();
}

IPTCMetadata.prototype                          = new Metadata();
IPTCMetadata.prototype.constructor              = IPTCMetadata;
IPTCMetadata.prototype.loadFromFile             = IPTCMetadata_loadFromFile;
IPTCMetadata.prototype.loadFromMedia            = IPTCMetadata_loadFromMedia;
IPTCMetadata.prototype.blankDocument            = IPTCMetadata_blank;

//
IPTCMetadata.prototype.getVersion               = IPTC_getVersion;
IPTCMetadata.prototype.setVersion               = IPTC_setVersion;

IPTCMetadata.prototype.getObjectTypeReference   = IPTC_getObjectTypeReference;
IPTCMetadata.prototype.setObjectTypeReference   = IPTC_setObjectTypeReference;

IPTCMetadata.prototype.getObjectAttributeReference = IPTC_getObjectAttributeReference;
IPTCMetadata.prototype.setObjectAttributeReference = IPTC_setObjectAttributeReference;
IPTCMetadata.prototype.addObjectAttributeReference = IPTC_addObjectAttributeReference;
IPTCMetadata.prototype.setObjectAttributeReferenceArray = IPTC_setObjectAttributeReferenceArray;

IPTCMetadata.prototype.getObjectName            = IPTC_getObjectName;
IPTCMetadata.prototype.setObjectName            = IPTC_setObjectName;

IPTCMetadata.prototype.getEditStatus            = IPTC_getEditStatus;
IPTCMetadata.prototype.setEditStatus            = IPTC_setEditStatus;

IPTCMetadata.prototype.getEditorialUpdate       = IPTC_getEditorialUpdate;
IPTCMetadata.prototype.setEditorialUpdate       = IPTC_setEditorialUpdate;

IPTCMetadata.prototype.getUrgency               = IPTC_getUrgency;
IPTCMetadata.prototype.setUrgency               = IPTC_setUrgency;

IPTCMetadata.prototype.getSubjectReference      = IPTC_getSubjectReference;
IPTCMetadata.prototype.addSubjectReference      = IPTC_addSubjectReference;
IPTCMetadata.prototype.setSubjectReference      = IPTC_setSubjectReference;
IPTCMetadata.prototype.setSubjectReferenceArray = IPTC_setSubjectReferenceArray;

IPTCMetadata.prototype.getCategory              = IPTC_getCategory;
IPTCMetadata.prototype.setCategory              = IPTC_setCategory;

IPTCMetadata.prototype.getSupplementalCategory  = IPTC_getSupplementalCategory;
IPTCMetadata.prototype.addSupplementalCategory  = IPTC_addSupplementalCategory;
IPTCMetadata.prototype.setSupplementalCategory  = IPTC_setSupplementalCategory;
IPTCMetadata.prototype.setSupplementalCategoryArray  = IPTC_setSupplementalCategoryArray;

IPTCMetadata.prototype.getFixtureIdentifier     = IPTC_getFixtureIdentifier;
IPTCMetadata.prototype.setFixtureIdentifier     = IPTC_setFixtureIdentifier;

IPTCMetadata.prototype.getKeywords              = IPTC_getKeywords;
IPTCMetadata.prototype.addKeywords              = IPTC_addKeywords;
IPTCMetadata.prototype.setKeywords              = IPTC_setKeywords;
IPTCMetadata.prototype.setKeywordsArray         = IPTC_setKeywordsArray;

IPTCMetadata.prototype.getContentLocation       = IPTC_getContentLocation;
IPTCMetadata.prototype.getContentLocationName   = IPTC_getContentLocationName;
IPTCMetadata.prototype.getContentLocationCode   = IPTC_getContentLocationCode;
IPTCMetadata.prototype.addContentLocation       = IPTC_addContentLocation;
IPTCMetadata.prototype.setContentLocation       = IPTC_setContentLocation;

IPTCMetadata.prototype.getReleaseDate           = IPTC_getReleaseDate;
IPTCMetadata.prototype.setReleaseDate           = IPTC_setReleaseDate;
IPTCMetadata.prototype.setReleaseTime           = IPTC_setReleaseTime;

IPTCMetadata.prototype.getExpirationDate        = IPTC_getExpirationDate;
IPTCMetadata.prototype.setExpirationDate        = IPTC_setExpirationDate;
IPTCMetadata.prototype.setExpirationTime        = IPTC_setExpirationTime;

IPTCMetadata.prototype.getSpecialInstructions   = IPTC_getSpecialInstructions;
IPTCMetadata.prototype.setSpecialInstructions   = IPTC_setSpecialInstructions;

IPTCMetadata.prototype.getActionAdvised         = IPTC_getActionAdvised;
IPTCMetadata.prototype.setActionAdvised         = IPTC_setActionAdvised;

IPTCMetadata.prototype.getReference             = IPTC_getReference;
IPTCMetadata.prototype.getReferenceService      = IPTC_getReferenceService;
IPTCMetadata.prototype.getReferenceDate         = IPTC_getReferenceDate;
IPTCMetadata.prototype.getReferenceNumber       = IPTC_getReferenceNumber;
IPTCMetadata.prototype.addReference             = IPTC_addReference;
IPTCMetadata.prototype.setReference             = IPTC_setReference;

IPTCMetadata.prototype.getDateCreated           = IPTC_getDateCreated;
IPTCMetadata.prototype.setDateCreated           = IPTC_setDateCreated;
IPTCMetadata.prototype.setTimeCreated           = IPTC_setTimeCreated;

IPTCMetadata.prototype.getDigitalCreationDate   = IPTC_getDigitalCreationDate;
IPTCMetadata.prototype.setDigitalCreationDate   = IPTC_setDigitalCreationDate;
IPTCMetadata.prototype.setDigitalCreationTime   = IPTC_setDigitalCreationTime;

IPTCMetadata.prototype.getOriginatingProgram    = IPTC_getOriginatingProgram;
IPTCMetadata.prototype.setOriginatingProgram    = IPTC_setOriginatingProgram;

IPTCMetadata.prototype.getProgramVersion        = IPTC_getProgramVersion;
IPTCMetadata.prototype.setProgramVersion        = IPTC_setProgramVersion;

IPTCMetadata.prototype.getObjectCycle           = IPTC_getObjectCycle;
IPTCMetadata.prototype.setObjectCycle           = IPTC_setObjectCycle;

IPTCMetadata.prototype.getByLine                = IPTC_getByLine;
IPTCMetadata.prototype.getByLineWriter          = IPTC_getByLineWriter;
IPTCMetadata.prototype.getByLineTitle           = IPTC_getByLineTitle;
IPTCMetadata.prototype.setByLine                = IPTC_setByLine;
IPTCMetadata.prototype.addByLine                = IPTC_addByLine;
IPTCMetadata.prototype.setByLineWriter          = IPTC_setByLineWriter;
IPTCMetadata.prototype.setByLineTitle           = IPTC_setByLineTitle;


IPTCMetadata.prototype.getCity                  = IPTC_getCity;
IPTCMetadata.prototype.setCity                  = IPTC_setCity;

IPTCMetadata.prototype.getSublocation           = IPTC_getSublocation;
IPTCMetadata.prototype.setSublocation           = IPTC_setSublocation;

IPTCMetadata.prototype.getState                 = IPTC_getState;
IPTCMetadata.prototype.setState                 = IPTC_setState;

IPTCMetadata.prototype.getCountryCode           = IPTC_getCountryCode;
IPTCMetadata.prototype.setCountryCode           = IPTC_setCountryCode;

IPTCMetadata.prototype.getCountryName           = IPTC_getCountryName;
IPTCMetadata.prototype.setCountryName           = IPTC_setCountryName;

IPTCMetadata.prototype.getOriginalTransmissionReference = IPTC_getOriginalTransmissionReference;
IPTCMetadata.prototype.setOriginalTransmissionReference = IPTC_setOriginalTransmissionReference;

IPTCMetadata.prototype.getHeadline              = IPTC_getHeadline;
IPTCMetadata.prototype.setHeadline              = IPTC_setHeadline;

IPTCMetadata.prototype.getCredit                = IPTC_getCredit;
IPTCMetadata.prototype.setCredit                = IPTC_setCredit;

IPTCMetadata.prototype.getSource                = IPTC_getSource;
IPTCMetadata.prototype.setSource                = IPTC_setSource;

IPTCMetadata.prototype.getCopyrightNotice       = IPTC_getCopyrightNotice;
IPTCMetadata.prototype.setCopyrightNotice       = IPTC_setCopyrightNotice;

IPTCMetadata.prototype.getContact               = IPTC_getContact;
IPTCMetadata.prototype.setContact               = IPTC_setContact;
IPTCMetadata.prototype.setContactArray          = IPTC_setContactArray;
IPTCMetadata.prototype.addContact               = IPTC_addContact;

IPTCMetadata.prototype.getCaption               = IPTC_getCaption;
IPTCMetadata.prototype.setCaption               = IPTC_setCaption;

IPTCMetadata.prototype.getWriter                = IPTC_getWriter;
IPTCMetadata.prototype.addWriter                = IPTC_addWriter;
IPTCMetadata.prototype.setWriter                = IPTC_setWriter;
IPTCMetadata.prototype.setWriterArray           = IPTC_setWriterArray;

IPTCMetadata.prototype.getImageType             = IPTC_getImageType;
IPTCMetadata.prototype.setImageType             = IPTC_setImageType;

IPTCMetadata.prototype.getImageOrientation      = IPTC_getImageOrientation;
IPTCMetadata.prototype.setImageOrientation      = IPTC_setImageOrientation;

IPTCMetadata.prototype.getLanguageIdentifier    = IPTC_getLanguageIdentifier;
IPTCMetadata.prototype.setLanguageIdentifier    = IPTC_setLanguageIdentifier;

IPTCMetadata.prototype.getAudioType             = IPTC_getAudioType;
IPTCMetadata.prototype.setAudioType             = IPTC_setAudioType;

IPTCMetadata.prototype.getAudioSamplingRate     = IPTC_getAudioSamplingRate;
IPTCMetadata.prototype.setAudioSamplingRate     = IPTC_setAudioSamplingRate;

IPTCMetadata.prototype.getAudioSamplingResolution = IPTC_getAudioSamplingResolution;
IPTCMetadata.prototype.setAudioSamplingResolution = IPTC_setAudioSamplingResolution;

IPTCMetadata.prototype.getAudioDuration         = IPTC_getAudioDuration;
IPTCMetadata.prototype.setAudioDuration         = IPTC_setAudioDuration;

IPTCMetadata.prototype.getAudioOutcue           = IPTC_getAudioOutcue;
IPTCMetadata.prototype.setAudioOutcue           = IPTC_setAudioOutcue;

IPTCMetadata.prototype._setDateTimeValue        = IPTC_setDateTimeValue;
