/*+
 * Metadata.ms
 *
 *  Contains the Metadata class which implements searching, getting values
 *  and setting values for metadata xml documents.
 *
 *  See method descriptions below.
 *
-*/

#include "sys:metadata_platform.ms"


/* ----------------------------------------------------------------------------
 *  Metadata
 *
 *  construct a new Metadata object with the specified file type and
 *  datatype.
 *
 *  validate    - (boolean) true to validate (default)
 *
 *  example:
 *      // load metadata from image.
 *      metadata = new Metadata("image.jpg", "IPTC", true);
 -----------------------------------------------------------------------------*/
function Metadata(validate)
{
    this._init(validate);
}

/* ----------------------------------------------------------------------------
 *  Metadata_loadFromXML
 *
 *  replace the document contents with the document in string.
 *
 *  str         - (string) the string containing xml data.
 *
 *  example:
 *      // load metadata from image.
 *      metadata = new Metadata();
 *      metadata.loadFromXML(xmlDoc);
 -----------------------------------------------------------------------------*/
function Metadata_loadFromXML(str)
{
    this._clear();
    this.dom.loadXML(str);
    Metadata_handleValidationErr(this.dom.parseError);
}

/* ----------------------------------------------------------------------------
 *  Metadata_loadFromFile
 *
 *  replace the document contents with the document in string.
 *
 *  fileName    - (string) the fileName.
 *  dataType    - (string) the data type to load
 *
 *  example:
 *      // load metadata from image.
 *      metadata = new Metadata();
 *      metadata.loadFromXML(xmlDoc);
 -----------------------------------------------------------------------------*/
function Metadata_loadFromFile(fileName, dataType)
{
    this._clear();
    var f = new File(fileName);
    if (!f.exists())
        error("File " + fileName + " does not exist.");

    var meta = new _MR_Metadata(fileName);
    var xmlData = meta.load(dataType);
    if (xmlData != null)
    {
         this.dom.loadXML(xmlData);
         Metadata_handleValidationErr(this.dom.parseError);
    }
}

/* ----------------------------------------------------------------------------
 *  Metadata_loadFromFile
 *
 *  replace the document contents with the document in string.
 *
 *  fileName    - (string) the fileName.
 *  dataType    - (string) the data type to load
 *
 *  example:
 *      // load metadata from image.
 *      metadata = new Metadata();
 *      metadata.loadFromXML(xmlDoc);
 -----------------------------------------------------------------------------*/
function Metadata_loadFromMedia(image, dataType)
{
    this._clear();
    var doc = image.getMetadata(dataType);
    if (doc != null)
    {
        this.dom.loadXML(doc);
        Metadata_handleValidationErr(this.dom.parseError);
   	}
}

/* ----------------------------------------------------------------------------
 *  Metadata_match  this.match(regExp)
 *
 *  return a list of node values matching regExp.
 *
 *  regExp      - (string/regexp) expression to search for
 *  returns     - (array) this list of matches
 *
 -----------------------------------------------------------------------------*/
function Metadata_match(regExpr)
{
    var results = [];
    return Metadata_appendMatching(this.dom, regExpr, results);
}

/* ----------------------------------------------------------------------------
 *  Metadata_match  this.search(regExp)
 *
 *  return a list of node names matching regExp.
 *
 *  regExp      - (string/regexp) expression to search for
 *  returns     - (array) this list of matches
 *
 -----------------------------------------------------------------------------*/
function Metadata_search(regExpr)
{
    var results = [];
    return Metadata_appendSearch(this.dom, regExpr, results);
}

/* ----------------------------------------------------------------------------
 *  Metadata_getAllValues  this.getAllValues(key)
 *
 *  return valArray with the value of all nodes matching key appended.
 *
 *  key         - (string) the key for this node
 *  returns     - (array) the value
 *
 -----------------------------------------------------------------------------*/
function Metadata_getAllValues(key)
{
    var vals = [];
    var nodeArray = this.dom.getElementsByTagName(key);
    for (var i = 0; i < nodeArray.length; i++)
        vals = Metadata_getNodeValue(nodeArray.item(i), vals);

    if (vals.length == 0)
        vals = vals.concat("");

    return vals;
}

/* ----------------------------------------------------------------------------
 *  Metadata_getObject  this.getValue(key)
 *
 *  return object with the value of all nodes matching key appended.
 *
 *  key         - (string) the key for this node
 *  returns     - (object) the value
 *
 -----------------------------------------------------------------------------*/
function Metadata_getValue(key)
{
    if (!this.isEmpty())
    {
        var node = this.nodeFromPath(key, false);
        if (node != null)
            return Metadata_getNodeValue(node);
    }
    return null;
}

/* ----------------------------------------------------------------------------
 * Metadata_setMatchingValues this.setMatchingValues(key, value)
 *
 ----------------------------------------------------------------------------*/
function Metadata_setMatchingValues(key, value)
{
    var path = key.split("/");
    var searchNode = path.pop();
    var root = null;

    if (path.length > 0)
        root = this._nodeFromArray(path, false);
    else
        root = this.dom.documentElement;

    if (root == null)
        return 0;

    var nodeArray = root.getElementsByTagName(searchNode);

    if (nodeArray.length)
    {
        // replace the existing nodes
        for (var i = 0; i < nodeArray.length; i++)
        {
            var node = this.dom.createElement(key);
            Metadata_setNodeContents(this.dom, node, value, attrs);
            nodeArray.item(i).parentNode.replaceChild(node, nodeArray.item(i));
        }
    }

    return nodeArray.length;
}

/* ----------------------------------------------------------------------------
 *  Metadata_setValue this.setValue(key, value)
 *
 *  set the contents of the node matching key to value. If there are no matching
 *  nodes, a node is added.
 *
 *  key         - (string) the key or path
 *  value       - (any value) the value
 *
 *  examples:
 *
 *  // set a simple value. Create or replace the specified node.
 *  // the root node name (IPTC below) is implied in key, but it may be specified.
 *  doc.setValue("IPTC/NodeName", "NodeValue");
 *  doc.setValue("NodeName", "NodeValue");
 *
 *  //  Create node below SubNode with NodeValue
 *  doc.setValue("SubNode/NodeName", "NodeValue");
 *
 -----------------------------------------------------------------------------*/
function Metadata_setValue(key, value, attrs)
{
    if (this.isEmpty())
        error("Cannot set values for an empty document.");
    if (value == null)
        error("Cannot setValue with null or undefined value.");

    //  empty strings and empty arrays force removal...
    var createNode = true;
    if (value.length != null && value.length == 0)
        createNode = false;

    //  also, if we an array with a single empty element, force removal
    if (createNode)
    {
        if (typeof(value) == "object" && value.length == 1 && value[0] == "")
            createNode = false;
    }

    // add or find the node.
    var node = this.nodeFromPath(key, createNode);

    if (createNode)
    {
        if (node.hasChildNodes()) {
            var kids = node.childNodes;
            //print ("\t#kids of " + node.nodeName + " is " + kids.length + "\n");
            while (kids.length)
            {
                //print("\tremoving " + kids.item(0).nodeName + "\n");
                node.removeChild(kids.item(0));
                //print("\tremoved\n");
            }
        }
        Metadata_setNodeContents(this.dom, node, value, attrs);
    }
    else
    {
        if (node != null)
        {
            //print("removing node: " + node.nodeName + "\n");
            node.parentNode.removeChild(node);
        }
    }
}

/* ----------------------------------------------------------------------------
 *  Metadata_nodeFromPath this.nodeFromPath(key, value)
 *
 *  set the contents of nodes matching key to value.
 *
 *  key         - (string) the key
 *  value       - (any value) the value
 *
 -----------------------------------------------------------------------------*/
function Metadata_nodeFromPath(pathString, create)
{
    return this._nodeFromArray(pathString.split("/"), create);
}

/* ----------------------------------------------------------------------------
 *  Metadata_isEmpty this.isEmpty()
 *
 *  return true if the document is empty.
 *
 *  returns     - true if empty
 *
 -----------------------------------------------------------------------------*/
function Metadata_isEmpty()
{
    return !this.dom.hasChildNodes();
}

/* ----------------------------------------------------------------------------
 *  Metadata_getDOM this.getDOM()
 *
 *  return the dom.
 *
 *  returns     - dom
 *
 -----------------------------------------------------------------------------*/
function Metadata_getDOM()
{
    return this.dom;
}

function Metadata_setValidates(validates)
{
    if (validates == null)
        validates = false;
    this.doValidation = validates;
}

function Metadata_nodeFromPathArray(path, create)
{
    var root = null;
    if (this.dom.documentElement.nodeName == path[0])
        path.shift();

    root = this.dom.documentElement;

    var len = path.length;
    for (var i = 0; i < len; i++)
    {
        var t = root.firstChild;
        while(t)
        {
            if (t.nodeName == path[i])
                break;
            //print("check failed: " + t.nodeName + " against " + path[i] + "\n");
            t = t.nextSibling;
        }

        if (t == null)
        {
            if (create)
            {
                //print("creating node: " + path[i] + "\n");
                var n = this.dom.createElement(path[i]);
                root.appendChild(n);
                root = n;
            }
            else
            {
                return null;
            }
        }
        else
        {
            root = t;
        }
    }

    //print ("\troot = " + (root ? root.nodeName : "undefined") + "\n");

    return root;
}

function Metadata_init(validate)
{
    if (validate == null)
        validate = true;
    this._createDOM(validate);
}

//
//  prototypes
//

Metadata.prototype.constructor          = Metadata;
Metadata.prototype._init                = Metadata_init;
Metadata.prototype.loadFromXML          = Metadata_loadFromXML;
Metadata.prototype._loadFromFile        = Metadata_loadFromFile;
Metadata.prototype._loadFromMedia       = Metadata_loadFromMedia;
Metadata.prototype.search               = Metadata_search;
Metadata.prototype.match                = Metadata_match;
Metadata.prototype.getValue             = Metadata_getValue;
Metadata.prototype.getAllValues         = Metadata_getAllValues;
Metadata.prototype.isEmpty              = Metadata_isEmpty;
Metadata.prototype.getDOM               = Metadata_getDOM;
Metadata.prototype.getXML               = Metadata_getXML;
Metadata.prototype.validate             = Metadata_validate;
Metadata.prototype.setValidates         = Metadata_setValidates;
Metadata.prototype.setValue             = Metadata_setValue;
Metadata.prototype.setMatchingValues    = Metadata_setMatchingValues;
Metadata.prototype.nodeFromPath         = Metadata_nodeFromPath;
Metadata.prototype._nodeFromArray       = Metadata_nodeFromPathArray;
Metadata.prototype.toString             = Metadata_getXML;
Metadata.prototype._createDOM           = Metadata_createDOM;
Metadata.prototype._clear               = Metadata_clear;


/* ----------------------------------------------------------------------------
 *  Metadata_setNodeContents (private)
 *
 *  set the contents of node to value.
 *
 *  dom         - (object) the dom object
 *  node        - (node) the node to set the value for
 *  value       - (any value) the value
 *  attr        - (attributes) the attributes (can be null)
 *
 -----------------------------------------------------------------------------*/
function Metadata_setNodeContents(dom, node, value, attr)
{
    //  distinguish between object value and simple value
    if (value == null)
        return;

    if (typeof(value) == "object")
    {
        //  is it an array?
        //print("node:" + node.nodeName + ", obj length:" + value.length + "\n");
        if (value.length != null) {
            //  add all the elements

            //print("setting object\n");
            for (var val in value) {
                var n = dom.createElement("element");
                Metadata_setNodeContents(dom, n, value[val], attr);
                node.appendChild(n);
            }
        } else{ // its an object
            for (var val in value)
            {
                var n = dom.createElement(val);
                Metadata_setNodeContents(dom, n, value[val], attr);
                node.appendChild(n);
            }
        }
    } else { // its a value
        //print("setting text:" + value + "\n");
        if (attr != null) {
            for (prop in attr)
                node.setAttribute(prop, attr[prop]);
        }

        var tnode = null;
        var sub = node.firstChild;
        while (sub) {
            if (sub.nodeType == Node.TEXT_NODE) {
                tnode = sub;
                break;
            }
            sub = sub.nextSibling;
        }
        var n = dom.createTextNode(value.toString());
        if (sub == null) {
            node.appendChild(n);
        } else {
            sub.parentNode.replaceChild(n, sub);
        }
    }
#if 0
        //print("obj length:" + value.length + "\n");
        if (value.length != null) {
            //  add all the elements

            //print("setting object\n");
            for (var val in value) {
                //print("got val:" + val + "\n");
                //  is the element a key/value object ?
                if (value[val].key != null) {
                    var n = dom.createElement(value[val].key);
                    Metadata_setNodeContents(dom, n, value[val].value, attr);
                } else {
                    //print("setting array element:" + value[val] + "\n");
                    var n = dom.createElement("element");
                    Metadata_setNodeContents(dom, n, value[val], attr);
                }
            }
        } else {
            var n = dom.createElement(value.key);
            Metadata_setNodeContents(dom, n, value.value, attr);
            node.appendChild(n);
        }
    } else {
        if (attr != null) {
            for (prop in attr)
                node.setAttribute(prop, attr[prop]);
        }

        var tnode = null;
        var sub = node.firstChild;
        while (sub) {
            if (sub.nodeType == Node.TEXT_NODE) {
                tnode = sub;
                break;
            }
            sub = sub.nextSibling;
        }
        var n = dom.createTextNode(value.toString());
        if (sub == null) {
            node.appendChild(n);
        } else {
            sub.parentNode.replaceChild(n, sub);
        }
    }
#endif
}

/* ----------------------------------------------------------------------------
 *  Metadata_getNodeObject  (private)
 *
 *  return object with the value of this node. The key is the node name.
 *  This follows our convention that array elements are all named "element".
 *  Nodes are processed accordingly, with "element" nodes assembled into an
 *  array and non "element" nodes as object params.
 *
 *  node        - (dom.Node) expression to search for
 *  parent      - (array) the parent object
 *  returns     - (object) the value
 *
 -----------------------------------------------------------------------------*/
function Metadata_getNodeValue(node)
{
    if (node.hasChildNodes())
    {
        var nodeObj;
        var n = node.firstChild;
        var isArray = n.nodeName == "element";
        if (isArray)
            nodeObj = [];
        else
            nodeObj = new Object();
        while(n != null)
        {
            if (n.nodeType == Node.TEXT_NODE)
            {
                return n.nodeValue;
            }
            else if (n.nodeType == Node.ELEMENT_NODE)
            {
                if (isArray)
                {
                    if (n.nodeName != "element")
                        error("wrong format for document");
                    nodeObj.push(Metadata_getNodeValue(n));
                }
                else
                {
                    nodeObj[n.nodeName] = Metadata_getNodeValue(n);
                }
            }

            n = n.nextSibling;
        }
        return nodeObj;
    }
    return null;
}

/* ----------------------------------------------------------------------------
 *  Metadata_appendMatching (private)
 *
 *  append node values matching expr to the list.
 *
 *  doc         - (object) the dom object
 *  expr        - (string/regex) the expression to search for
 *
 -----------------------------------------------------------------------------*/
function Metadata_appendMatching(doc, expr, list)
{
    if (doc.hasChildNodes())
    {
        var theNode = doc.firstChild;
        while(theNode != null)
        {
            if (theNode.nodeType == Node.TEXT_NODE)
            {
                if (theNode.nodeValue.match(expr))
                    list = list.concat(theNode.nodeValue);
            }
            else
            {
                list = Metadata_appendMatching(theNode, expr, list);
            }
            theNode = theNode.nextSibling;
        }
    }
    return list;
}

/* ----------------------------------------------------------------------------
 *  Metadata_appendSearch (private)
 *
 *  append node names matching expr to the list.
 *
 *  doc         - (object) the dom object
 *  expr        - (string/regex) the expression to search for
 *  list        - (array) the list of matching node names
 *
 -----------------------------------------------------------------------------*/
function Metadata_appendSearch(doc, expr, list)
{
    if (doc.hasChildNodes())
    {
        var theNode = doc.firstChild;
        while(theNode != null)
        {
            if (theNode.nodeType == Node.TEXT_NODE)
            {
                if (theNode.nodeValue.search(expr) != -1)
                    list = list.concat(theNode.nodeValue);
            }
            else
            {
                list = Metadata_appendSearch(theNode, expr, list);
            }
            theNode = theNode.nextSibling;
        }
    }
    return list;
}

