/*==============================================================================
*
* File:         TextResponse.ms
*
* Contains:
* Class for text responses (using resp.setObject()).
*
* Written by:   josh_devan
*
* Copyright:    Copyright (C) 2002 Equilibrium, Inc. All rights reserved.
*
*=============================================================================*/

// ----------------------------------- prototype -------------------------------

TextResponse.prototype.getType      = TextResponseGetType;
TextResponse.prototype.setText      = TextResponseSetText;
TextResponse.prototype.getText      = TextResponseGetText;
TextResponse.prototype.append       = TextResponseAppend;
TextResponse.prototype.setTextType  = TextResponseSetTextType;
TextResponse.prototype.getTextType  = TextResponseGetTextType;
TextResponse.prototype._MR_save     = TextResponseSave;

// Text types
TextResponse.NumTypes   = 4;

TextResponse.TypePlain  = 0;
TextResponse.TypeHtml   = 1;
TextResponse.TypeXml    = 2;
TextResponse.TypeJson   = 3;

TextResponse.typeArray = 
{
    {mimeType: "text/plain; charset=utf-8", ext: ".txt"},
    {mimeType: "text/html; charset=utf-8", ext: ".html"},
    {mimeType: "text/xml; charset=utf-8", ext: ".xml"},
    {mimeType: "application/json; charset=utf-8", ext: ".json"},
};


// ----------------------------------- methods ---------------------------------


/*----------------------------------------------------------------------------
*
* Function: TextResponse
*
* Description:
* Constructor for TextResponse object.
*
* Parameters:
* number textType       -> One of the types listed above.
*
* Return:
* None
*
------------------------------------------------------------------------------*/
function TextResponse(textType)
{
    // Default to plain text.
    if (textType == null)
        textType = TextResponse.TypePlain;
    
    if (!((textType >= 0) && (textType < TextResponse.NumTypes)))
        throw "Invalid text type: " + textType + ".";
    
    this.textType = textType;
    this.text = "";
}

/*----------------------------------------------------------------------------
*
* Function: TextResponseGetType
*
* Description:
* Returns the type of this object (always "TextResponse").
*
* Parameters:
* None
*
* Return:
* string                <- Type of object ("TextResponse").
*
------------------------------------------------------------------------------*/
function TextResponseGetType()
{
    return "TextResponse";
}

/*----------------------------------------------------------------------------
*
* Function: TextResponseGetText
*
* Description:
* Returns the text string associated with this object.
*
* Parameters:
* None
*
* Return:
* string                <- Text string.
*
------------------------------------------------------------------------------*/
function TextResponseGetText()
{
    return this.text;
}

/*----------------------------------------------------------------------------
*
* Function: TextResponseSetText
*
* Description:
* Sets the text string associated with this object.
*
* Parameters:
* string text           -> New text string.
*
* Return:
* None
*
------------------------------------------------------------------------------*/
function TextResponseSetText(text)
{
    this.text = text;
}

/*----------------------------------------------------------------------------
*
* Function: TextResponseAppend
*
* Description:
* Appends the given string to the text associated with this object.
*
* Parameters:
* string text           -> Text string to append.
*
* Return:
* None
*
------------------------------------------------------------------------------*/
function TextResponseAppend(text)
{
    this.text += text;
}

/*----------------------------------------------------------------------------
*
* Function: TextResponseGetTextType
*
* Description:
* Returns the text type associated with this object.
*
* Parameters:
* None
*
* Return:
* number                <- Text type.
*
------------------------------------------------------------------------------*/
function TextResponseGetTextType()
{
    return this.textType;
}

/*----------------------------------------------------------------------------
*
* Function: TextResponseSetTextType
*
* Description:
* Sets the text type associated with this object (must be one of the defines
* above).
*
* Parameters:
* number textType       -> Text type.
*
* Return:
* None
*
------------------------------------------------------------------------------*/
function TextResponseSetTextType(textType)
{
    if (!((textType >= 0) && (textType < TextResponse.NumTypes)))
        throw "Invalid text type: " + textType + ".";
        
    this.textType = textType;
}

/*----------------------------------------------------------------------------
*
* Function: TextResponseSave
*
* Description:
* Implements _MR_save interface to allow TextResponse objects to be used as
* response objects.
*
* In addition to the mandatory 'name' parameter, the saveParams object for
* TextResponseSave accepts the following fields:
*
*   ext:        File extension (overrides default file extension for text type)
*   mimeType:   MIME type (overrides default MIME type for text type)
*
* Parameters:
* object saveParams     -> Save parameters.
*
* Return:
* object                <- Save results object with 'path' and 'mimeType' 
*                          fields as defined by _MR_save interface.
*
------------------------------------------------------------------------------*/
function TextResponseSave(saveParams)
{
    var saveResults;

    // Determine file path
    saveResults.path = saveParams.name;
    if (saveParams.ext)
    {   
        saveResults.path += saveParams.ext;
    }
    else
    {   
        var ext = (new File(saveResults.path)).getFileExtension();
        if (!ext)
            saveResults.path += TextResponse.typeArray[this.textType].ext;
    }

    // Determine MIME type    
    if (saveParams.mimeType != null)
        saveResults.mimeType = saveParams.mimeType;
    else
        saveResults.mimeType = TextResponse.typeArray[this.textType].mimeType;

    // Save text to output file
    var saveFile = new File(saveResults.path);
    saveFile.clear();
    saveFile.write(this.text);
    saveFile.close();

    return saveResults;
}

// -----------------------------------------------------------------------------
// ---------------------------------- utilities --------------------------------
// -----------------------------------------------------------------------------

var mush = false; try { req == undefined; } catch(e) { mush = true; }

//-----------------------------------------------------------------------------

function MushPrintLn(str)
{
    if (mush)
        print(str + "\n");
}
//-----------------------------------------------------------------------------

function SetTextResponse(responseText)
{
    if (mush) { MushPrintLn(responseText); return; }
    var strObject = new TextResponse(TextResponse.TypePlain); 
    strObject.setText(responseText); 
    resp.setObject(strObject, RespType.Streamed);
}
//-----------------------------------------------------------------------------

function setTextResponse(responseText)
{
    SetTextResponse(responseText);
}
//-----------------------------------------------------------------------------

function SetHTMLResponse(responseHtml)
{
    if (mush) { MushPrintLn(responseHtml); return; }
    var strObject = new TextResponse(TextResponse.TypeHtml); 
    strObject.setText(responseHtml); 
    resp.setObject(strObject, RespType.Streamed);
}
//-----------------------------------------------------------------------------

function SetJSONResponse(responseJson)
{
    if (mush) { MushPrintLn(responseJson); return; }
    var strObject = new TextResponse(TextResponse.TypeJson);
    strObject.setText(responseJson);
    resp.setObject(strObject, RespType.Streamed);
}
//-----------------------------------------------------------------------------

function IsTextResponseEqualTo(theText)
{
    try
    {
        var respObject = resp.getObject();
        var text = respObject.getText(); 
        return text == theText;
    }
    catch (ex)
    {
        return false;
    }
}
//-----------------------------------------------------------------------------
