#link "XMPToolkit.mdv"

// XMPMetadata.ms

function logWarning(msg)
{
	var f = new LogFile("XMPMetadata.log");
	f.appendLine(msg, LogFile.SeverityWarning);
}


/* ----------------------------------------------------------------------------
 *  XMPMetadata
 *
 *  construct a new, empty Metadata object.
 *
 *  example:
 *      // load metadata from image.
 *      metadata = new Metadata();
 -----------------------------------------------------------------------------*/
function XMPMetadata()
{
	this.xmp = new XMPToolkit();
}


/* ----------------------------------------------------------------------------
 *  XMPMetadata_blank
 *
 *  create blank XMP document, or make an existing document blank.
 *	Note that the root rdf:RDF element is created, all added elements must
 *	be inside this node.
 *
 *  example:
 *      // load metadata from image.
 *      metadata = new XMPMetadata();
 *      metadata.blankDocument();
 -----------------------------------------------------------------------------*/
function XMPMetadata_blank
{
	this.xmp.clear();
}


/* ----------------------------------------------------------------------------
 *  XMPMetadata_loadFromFile
 *
 *  load XMPMetadata object from the file.
 *
 *  fileName    - (string) the name of the image file to get the data from
 *
 *  example:
 *      // load metadata from image.
 *      metadata = new XMPMetadata();
 *      metadata.loadFromFile("image.jpg");
 -----------------------------------------------------------------------------*/
function XMPMetadata_loadFromFile(fileName)
{
	this.xmp.clear();
	this.xmp = new XMPToolkit(fileName);
}


/* ----------------------------------------------------------------------------
 *  XMPMetadata_loadFromMedia
 *
 *  load XMPMetadata object from the media.
 *
 *  media       - the media (object) to get the metadata from.
 *
 *  example:
 *      // load metadata from image.
 *      var m = new Media();
 *      m.load(name @ "image.jpg");
 *      metadata = new XMPMetadata();
 *      metadata.loadFromMedia(m);
 -----------------------------------------------------------------------------*/
function XMPMetadata_loadFromMedia(media)
{
	var doc = image.getMetadata("XMP");
	if (doc != null)
		this.xmp.parse(doc);
}


/* ----------------------------------------------------------------------------
 *  XMPMetadata_loadFromDiskFile
 *
 *  load xmp data from a disk (xml) file.
 *
 *  path       - the path to the file.
 *
 *  example:
 *      // load metadata from image.
 *      metadata = new XMPMetadata();
 *      metadata.loadFromDiskFile("file.xmp.xml");
 -----------------------------------------------------------------------------*/
function XMPMetadata_loadFromDiskFile(path)
{
	this.xmp.clear();
	this.xmp = new XMPToolkit(path);	// XMPToolkit is able to read .xml by itself
}


/* ----------------------------------------------------------------------------
 *  XMPMetadata_getSingleValueForNode
 *
 *  get a single value for the specified element. If there are enclosed
 *	elements, the node values are concatenated with newlines.
 *
 *  elementName       - the element to return values and subvalues from.
 *
 -----------------------------------------------------------------------------*/
function XMPMetadata_getSingleValueForNode(elementName)
{
	return xmp.enumerateWithPrefix(elementName);
}


/* ----------------------------------------------------------------------------
 *  XMPMetadata_getTagValue
 *
 *  get a single value for the element tag name. The tag should use a "standard"
 *	namespace alias (see XMPMetadataBaseNSTable for values). Before searching
 *	the document, the tagName is mapped into the documents namespace alias 
 *	space from the "standard" space.
 *
 *  rdf:Bag and rdf:Seq element values are returned as a single string with 
 *  newlines separating the individual values.
 *
 *  tagName       - the tag name using standard namespace aliases.
 *
 -----------------------------------------------------------------------------*/
function XMPMetadata_getTagValue(tagName)
{
	var s = null;
	if (XMPToolkit.isSimple(this.xmp.getPropertyTypeWithPrefix(tagName)))
	{
		s = this.xmp.getPropertyWithPrefix(tagName);
	}
	else
	{
		s = this.xmp.enumerateWithPrefix(tagName);
		if (s.indexOf("\n") >= 0)
			logWarning("multiple nodes found for " + tagName);
	}
	
	return s;
}


/* ----------------------------------------------------------------------------
 *  XMPMetadata_setTagValue
 *
 *  Set the value for the simple element designated by tagName.
 *
 *  value       - (string) value to set.
 *	tagName		- location for value. The tagName must use one of the namespace
 *					aliases defined in XMPMetadataBaseNSTable.
 *
 -----------------------------------------------------------------------------*/
function XMPMetadata_setTagValue(value, tagName)
{
	var type = this.xmp.getPropertyTypeWithPrefix(tagName);
	if (XMPToolkit.isArray(type))
		tagName += "[1]";		// old API here allowed setting of single-element arrays w/o an index, so fudge it here
	this.xmp.setPropertyWithPrefix(tagName, value);
}


/* ----------------------------------------------------------------------------
 *  XMPMetadata_setTagBagValue
 *
 *  Set a multi-valued element usign the rdf:Bag construct. The value must be
 *	either an array or a new-line delimited string.
 *
 *  value       - (string) value to set.
 *	tagName		- location for value. The tagName must use one of the namespace
 *					aliases defined in XMPMetadataBaseNSTable.
 *
 -----------------------------------------------------------------------------*/
function XMPMetadata_setTagBagValue(value, tagName)
{
	var arr;
	if (typeof value == "string")
		arr = value.split("\n");
	else
		arr = value;
	
	this.xmp.deletePropertyWithPrefix(tagName);	// delete the existing array; we recreate it in its entirety below
	for (var i=0; i<arr.length; i++)
		this.xmp.appendToArrayWithPrefix(tagName, arr[i], XMPToolkit.kPropArrayIsUnordered);
}


/* ----------------------------------------------------------------------------
 *  XMPMetadata_setTagSeqValue
 *
 *  Set a multi-valued element usign the rdf:Seq construct. The value must be
 *	either an array or a new-line delimited string.
 *
 *  value       - (string) value to set.
 *	tagName		- location for value. The tagName must use one of the namespace
 *					aliases defined in XMPMetadataBaseNSTable.
 *
 -----------------------------------------------------------------------------*/
function XMPMetadata_setTagSeqValue(value, tagName)
{
	var arr;
	if (typeof value == "string")
		arr = value.split("\n");
	else
		arr = value;
	
	this.xmp.deletePropertyWithPrefix(tagName);	// delete the existing array; we recreate it in its entirety below
	for (var i=0; i<arr.length; i++)
		this.xmp.appendToArrayWithPrefix(tagName, arr[i], XMPToolkit.kPropArrayIsOrdered);
}


/*	Convert the object into a nicely formatted XML string
*/
function XMPMetadata_toString()
{
	return this.xmp.serialize();
}


/*	Turn an XML string into XMP metadata
*/
function XMPMetadata_loadFromXML(xml)
{
	this.xmp.parse(xml);
}


XMPMetadata.prototype.constructor			= XMPMetadata;
XMPMetadata.prototype.loadFromFile			= XMPMetadata_loadFromFile;
XMPMetadata.prototype.loadFromMedia			= XMPMetadata_loadFromMedia;
XMPMetadata.prototype.loadFromDiskFile		= XMPMetadata_loadFromDiskFile;
XMPMetadata.prototype.blankDocument			= XMPMetadata_blank;

XMPMetadata.prototype.getSingleValueForNode	= XMPMetadata_getSingleValueForNode;
XMPMetadata.prototype.getTagValue			= XMPMetadata_getTagValue;
XMPMetadata.prototype.setTagValue			= XMPMetadata_setTagValue;
XMPMetadata.prototype.setTagBagValue		= XMPMetadata_setTagBagValue;
XMPMetadata.prototype.setTagSeqValue		= XMPMetadata_setTagSeqValue;

// These were originally in the parent class of the old XMPMetadata object. Emulate them w/ XMPToolkit as well because SP Connector uses them.
XMPMetadata.prototype.toString				= XMPMetadata_toString;
XMPMetadata.prototype.getXML				= XMPMetadata_toString;
XMPMetadata.prototype.loadFromXML			= XMPMetadata_loadFromXML;
// TODO - should probably add the rest of the parent methods before final 3.9.8 release!
