// ============================================================================
//  color.ms
//
//  Utility object for color representation.
//
//  RgbColor
//      This object is constructed from a 24 bit value. It splits the color
//      into red, green, and blue components.
//
//      constructor:
//          RgbColor(value) - returns an RgbColor object constructed from value.
//      properties:
//          .red    - the red component. read or write. Range 0-255.
//          .green  - the green component. read or write. Range 0-255.
//          .blue   - the blue component. read or write. Range 0-255.
//      methods:
//          valueOf() - converts the red, green, and blue components back to
//                      a 24 bit value.
//          toString()- returns string representation of 24-bit value.
//      functions:
//          RgbColorFromRGB(red, green, blue)
//              construct RgbColor from the components.
//          RgbColorFromPct(red, green, blue)
//              construct RgbColor from component percentages (0-1).
//
//  Examples:
//      myColor = new RgbColor(0x1133aa);
//      print (myColor.red, myColor.green, myColor.blue);
//
//      myColor = new RgbColor();
//      myColor.red = 27;
//      myColor.green = 59;
//      myColor.blue = 255;
//      media = new Media();
//      media.makeCanvas(xs @ 100, ys @ 100, fillcolor @ myColor);
//
//  CmykColor
//      This object is constructed from a 32 bit value. It splits the color
//      into cyan, magenta, yellow, and black components.
//
//      constructor:
//          CmykColor(value) - returns an CmykColor object constructed from value.
//      properties:
//          .cyan   - the cyan component. read or write. Range 0-255.
//          .magenta- the magenta component. read or write. Range 0-255.
//          .yellow - the yellow component. read or write. Range 0-255.
//          .black  - the black component. read or write. Range 0-255.
//      methods:
//          valueOf() - converts the cyan, magenta, yellow, and black components
//                      back to a 32 bit value.
//          toString()- returns string representation of 32-bit value.
//      functions:
//          CmykColorFromCMYK(cyan, magenta, yellow, black)
//              construct CmykColor from the components.
//          CmykColorFromPct(cyan, magenta, yellow, black)
//              construct CmykColor from component percentages (0-1).
//
//  Examples:
//      myColor = new CmykColor();
//      myColor.cyan = 27;
//      myColor.yellow = 122;
//      myColor.magenta = 115;
//      myColor.black = 55;
//      media = new Media();
//      media.makeCanvas(xs @ 100, ys @ 100, fillcolor @ myColor);
//
// ============================================================================

function colorCheckRGB(r, g, b)
{
    if (r < 0 || r > 255)
        error("value for red is not in the range 0-255");
    else if (g < 0 || g > 255)
        error("value for green is not in the range 0-255");
    else if (b < 0 || b > 255)
        error("value for blue is not in the range 0-255");
}


function colorCheckCMYK(c, m, y, k)
{
    if (c < 0 || c > 255)
        error("value for cyan is not in the range 0-255");
    else if (m < 0 || m > 255)
        error("value for magenta is not in the range 0-255");
    else if (y < 0 || y > 255)
        error("value for yellow is not in the range 0-255");
    else if (k < 0 || k > 255)
        error("value for black is not in the range 0-255");
}

//  convert to value
function RgbColor_value()
{
    colorCheckRGB(this.red, this.green, this.blue);
    return (this.red << 16) | (this.green << 8) | this.blue;
}


//  RgbColor constructor
function RgbColor(color)
{
    this.red  = (color >> 16) & 0xff;
    this.green  = (color >> 8) & 0xff;
    this.blue  = color & 0xff;
}

//  prototypes for converting to values
RgbColor.prototype.valueOf = RgbColor_value;
RgbColor.prototype.toString = RgbColor_value;

//  Construct RgbColor from red, green, blue in the range 0-255
function RgbColorFromRGB(red, green, blue)
{
    colorCheckRGB(red, green, blue);
    c =  new RgbColor();
    c.red   = red;
    c.green = green;
    c.blue  = blue;
    return c;
}

//  Construct RgbColor from percentages
function RgbColorFromPct(red, green, blue)
{
    if (red < 0 || red > 1 || green < 0 || green > 1 || blue < 0 || blue > 1)
        error("RgbColorFromPct requires values between 0 and 1");
    return RgbColorFromRGB(red * 0xff, green * 0xff, blue * 0xff);
}

//  Constructor - construct cmyk components from value
function CmykColor(color)
{
    this.cyan  = (color >> 24) & 0xff;
    this.magenta  = (color >> 16) & 0xff;
    this.yellow  = (color >>  8) & 0xff;
    this.black  = color & 0xff;
}

//  compute value from components.
function CmykColor_value
{
    colorCheckCMYK(this.cyan, this.magenta, this.yellow, this.black);
    return (this.cyan << 24) | (this.magenta << 16) | (this.yellow << 8) | this.black;
}

//  protypes for value and string conversion
CmykColor.prototype.toString = CmykColor_value;
CmykColor.prototype.valueOf = CmykColor_value;

//  Constuct CmykColor from components
function CmykColorFromCMYK(cyan, magenta, yellow, black)
{
    colorCheckCMYK(cyan, magenta, yellow, black);
    CmykColor c = new CmykColor();
    c.cyan      = cyan;
    c.magenta   = magenta;
    c.yellow    = yellow;
    c.black     = black;
    return c;
}

//  Construct CmykColor from percentages
function CmykColorFromPct(cyan, magenta, yellow, black)
{
    if (cyan < 0 || cyan > 1 || magenta < 0 || magenta > 1 ||
            yellow < 0 || yellow > 1 || black < 0 || black > 1)
        error("CmykColorFromPct requires values between 0 and 1");
    return CmykColorFromCMYK(cyan * 0xff, magenta * 0xff, yellow * 0xff, black * 0xff);
}

