/*==============================================================================
*
* File:     dom.ms
*
* Contains:
*   contains the objects specified in the w3c ecma-script binding for DOM 
*   level-1 core
*   http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/ecma-script-language-binding.html
*
* Written by:   matt_butler
*
* Copyright:    Copyright (C) 2001 Equilibrium, Inc. All rights reserved.
*
*=============================================================================*/

/*----------------------------------------------------------------------------
*
* Class: DOMException
*
* Description:
* A dom exception
*
* Parameters:
* None
*
* Return:
* None
*
------------------------------------------------------------------------------*/
function DOMException()
{
    this.code = 0;
}

/*----------------------------------------------------------------------------
*
* Class: ExceptionCode
*
* Description:
* The code of an exception
*
* Parameters:
* None
*
* Return:
* None
*
------------------------------------------------------------------------------*/
function ExceptionCode()
{
}

ExceptionCode.INDEX_SIZE_ERR                = 1;
ExceptionCode.DOMSTRING_SIZE_ERR            = 2;
ExceptionCode.HIERARCHY_REQUEST_ERR         = 3;
ExceptionCode.WRONG_DOCUMENT_ERR            = 4;
ExceptionCode.INVALID_CHARACTER_ERR         = 5;
ExceptionCode.NO_DATA_ALLOWED_ERR           = 6;
ExceptionCode.NO_MODIFICATION_ALLOWED_ERR   = 7;
ExceptionCode.NOT_FOUND_ERR                 = 8;
ExceptionCode.NOT_SUPPORTED_ERR             = 9;
ExceptionCode.INUSE_ATTRIBUTE_ERR           = 10;

/*----------------------------------------------------------------------------
*
* Class: DOMImplementation
*
* Description:
* Info about this implementation
*
* Parameters:
* None
*
* Return:
* None
*
------------------------------------------------------------------------------*/
function DOMImplementation()
{
}

function DOMImplementationHasFeature(feature, version)
{
    if(version != "1.0")
        return false;
    if(feature.toUpperCase() != "XML")
        return false;
    return true;
}

DOMImplementation.prototype.hasFeature = DOMImplementationHasFeature;

/*----------------------------------------------------------------------------
*
* Class: NodeList
*
* Description:
* A list of nodes
*
* Parameters:
* None
*
* Return:
* None
*
------------------------------------------------------------------------------*/
function NodeList()
{
    this.list = [];
    this.length = 0;
}

function NodeListItem(index)
{
    if(index < 0)
        return null;
    if(index > this.list.length - 1)
        return null;
    return this.list[index];
}

function NodeListToString()
{
    var tmp = "[";
    var i;

    for(i = 0; i < this.list.length; i++)
    {
        tmp += this.list[i];
        tmp += ", ";
    }
    tmp += this.length
    tmp += "]";
    return tmp;
}

NodeList.prototype.item     = NodeListItem;
NodeList.prototype.toString = NodeListToString;

/*----------------------------------------------------------------------------
*
* Class: NamedNodeMap
*
* Description:
* An associative array of Nodes
*
* Parameters:
* None
*
* Return:
* None
*
------------------------------------------------------------------------------*/
function NamedNodeMap()
{
    this.map = new Object();
    this.length = 0;
}

function NamedNodeMapGetNamedItem(name)
{
    try
    {
        return this.map[name];
    }
    catch(e)
    {
        return null;
    }
}

function NamedNodeMapSetNamedItem(arg) 
{
    oldNode = this.map[arg.nodeName];
    this.map[arg.nodeName] = arg;
    if(!oldNode)
        this.length++;
    return oldNode;
}

function NamedNodeMapRemoveNamedItem(name) 
{
    if(!this.map[name])
    {
        de = new DOMException();
        de.code = ExceptionCode.NOT_FOUND_ERR;
        throw(de);
    }
    tmp = this.map[name];
    delete(this.map[name]);
    this.length--;
    return tmp;
}

function NamedNodeMapItem(index) 
{
    var i = 0;
    for(item in this.map)
    {
        if(i == index)
            return this.map[item];
        i++;
    }
    return null;
}

function NamedNodeMapToString()
{
    var tmp = "(";
    var i;

    for(i in this.map)
    {
        tmp += this.map[i].toString();
        tmp += ", ";
    }
    tmp += this.length
    tmp += ")";
    return tmp;
}

NamedNodeMap.prototype.getNamedItem     = NamedNodeMapGetNamedItem;
NamedNodeMap.prototype.setNamedItem     = NamedNodeMapSetNamedItem;
NamedNodeMap.prototype.removeNamedItem  = NamedNodeMapRemoveNamedItem;
NamedNodeMap.prototype.item             = NamedNodeMapItem;
NamedNodeMap.prototype.toString         = NamedNodeMapToString;

/*----------------------------------------------------------------------------
*
* Class: Node
*
* Description:
* The basic Node of the DOM tree
*
* Parameters:
* None
*
* Return:
* None
*
------------------------------------------------------------------------------*/
function Node()
{
    Node_init(this);
}

function NodeInsertBefore(newChild, refChild)
{
    var i;

    if((this.nodeType == Node.DOCUMENT_NODE) && (newChild.nodeType == Node.ELEMENT_NODE))
    {
        if(this.documentElement || (refChild != null))
        {
            de = new DOMException();
            de.code = ExceptionCode.HIERARCHY_REQUEST_ERR;
            throw(de);
        }
    }

    if(newChild.nodeType == Node.DOCUMENT_FRAGMENT_NODE)
    {
        var tmp = newChild.firstChild;
        while(tmp)
        {
            var nxt = tmp.nextSibling;
            this.insertBefore(tmp, refChild);
            tmp = nxt;
        }
        return newChild;
    }

    if(!this._checkChildNodeType(newChild))
    {
        de = new DOMException();
        de.code = ExceptionCode.HIERARCHY_REQUEST_ERR;
        throw(de);
    }

    if((newChild.ownerDocument != this.ownerDocument) && (newChild.ownerDocument != this))
    {
        de = new DOMException();
        de.code = ExceptionCode.WRONG_DOCUMENT_ERR;
        throw(de);
    }

    if(!refChild)
    {
        this.appendChild(newChild);
        return newChild;
    }

    if(newChild.parentNode)
        newChild.parentNode.removeChild(newChild);

    i = this._findChildIndex(refChild)
    if(i == -1)
    {
        de = new DOMException();
        de.code = ExceptionCode.NOT_FOUND_ERR;
        throw(de);
    }

    this.childNodes.list.splice(i, 0, newChild);
    this.childNodes.length = this.childNodes.list.length;
    this.firstChild = this.childNodes.item(0);
    this.lastChild = this.childNodes.item(this.childNodes.length - 1);
    newChild.parentNode = this;
    newChild.nextSibling = refChild;
    refChild.previousSibling = newChild;

    if(i > 0)
    {
        this.childNodes[i - 1].nextSibling = newChild;
        newChild.previousSibling = this.childNodes[i - 1];
    }
    else
        newChild.previousSibling = null;

    if((this.nodeType == Node.DOCUMENT_NODE) && (newChild.nodeType == Node.ELEMENT_NODE))
        this.documentElement = newChild;

    return newChild;
}

function NodeReplaceChild(newChild, oldChild) 
{
    var i;

    if(newChild.nodeType == Node.DOCUMENT_FRAGMENT_NODE)
    {
        var refChild = oldChild.nextSibling;
        this.removeChild(oldChild);
        this.insertBefore(newChild, refChild);
        return oldChild;
    }

    if(!this._checkChildNodeType(newChild))
    {
        de = new DOMException();
        de.code = ExceptionCode.HIERARCHY_REQUEST_ERR;
        throw(de);
    }

    if((newChild.ownerDocument != this.ownerDocument) && (newChild.ownerDocument != this))
    {
        de = new DOMException();
        de.code = ExceptionCode.WRONG_DOCUMENT_ERR;
        throw(de);
    }

    if(newChild == oldChild)
        return oldChild;

    if(newChild.parentNode)
        newChild.parentNode.removeChild(newChild);

    i = this._findChildIndex(oldChild);
    if(i == -1)
    {
        de = new DOMException();
        de.code = ExceptionCode.NOT_FOUND_ERR;
        throw(de);
    }

    this.childNodes.list[i] = newChild;
    this.firstChild = this.childNodes.item(0);
    this.lastChild = this.childNodes.item(this.childNodes.length - 1);
    newChild.parentNode = this;
    oldChild.parentNode = null;
    newChild.nextSibling = oldChild.nextSibling;
    newChild.previousSibling = oldChild.previousSibling;
    oldChild.nextSibling = null;
    oldChild.previousSibling = null;
    if(newChild.nextSibling)
        newChild.nextSibling.previousSibling = newChild;
    if(newChild.previousSibling)
        newChild.previousSibling.nextSibling = newChild;

    if(this.nodeType == Node.DOCUMENT_NODE && (newChild.nodeType == Node.ELEMENT_NODE))
        this.documentElement = newChild;

    return oldChild;
}

function NodeRemoveChild(oldChild) 
{
    var i;

    i = this._findChildIndex(oldChild);
    if(i == -1)
    {
        de = new DOMException();
        de.code = ExceptionCode.NOT_FOUND_ERR;
        throw(de);
    }

    this.childNodes.list.splice(i, 1);
    this.childNodes.length = this.childNodes.list.length;
    this.firstChild = this.childNodes.item(0);
    this.lastChild = this.childNodes.item(this.childNodes.length - 1);

    if(oldChild.nextSibling)
        oldChild.nextSibling.previousSibling = oldChild.previousSibling;
    if(oldChild.previousSibling)
        oldChild.previousSibling.nextSibling = oldChild.nextSibling;
    oldChild.parentNode = null;

    if((this.nodeType == Node.DOCUMENT_NODE) && (this.documentElement == oldChild))
        this.documentElement = null;

    return oldChild;
}

function NodeAppendChild(newChild) 
{
    if((this.nodeType == Node.DOCUMENT_NODE) && this.documentElement && (newChild.nodeType == Node.ELEMENT_NODE))
    {
        de = new DOMException();
        de.code = ExceptionCode.HIERARCHY_REQUEST_ERR;
        throw(de);
    }

    if(newChild.nodeType == Node.DOCUMENT_FRAGMENT_NODE)
    {
        var tmp = newChild.firstChild;
        while(tmp)
        {
            var nxt = tmp.nextSibling;
            this.appendChild(tmp);
            tmp = nxt;
        }
        return newChild;
    }

    if(!this._checkChildNodeType(newChild))
    {
        de = new DOMException();
        de.code = ExceptionCode.HIERARCHY_REQUEST_ERR;
        throw(de);
    }

    if((newChild.ownerDocument != this.ownerDocument) && (newChild.ownerDocument != this))
    {
        de = new DOMException();
        de.code = ExceptionCode.WRONG_DOCUMENT_ERR;
        throw(de);
    }

    if(newChild.parentNode)
        newChild.parentNode.removeChild(newChild);

    if(this.hasChildNodes())
    {
        oldLast = this.lastChild;
        newChild.previousSibling = oldLast;
        oldLast.nextSibling = newChild;
    }
    else
    {
        newChild.previousSibling = null;
    }

    newChild.nextSibling = null;
    newChild.parentNode = this;
    this.childNodes.list.push(newChild);
    this.childNodes.length = this.childNodes.list.length;
    this.firstChild = this.childNodes.item(0);
    this.lastChild = this.childNodes.item(this.childNodes.length - 1);
    
    if((this.nodeType == Node.DOCUMENT_NODE) && (newChild.nodeType == Node.ELEMENT_NODE))
        this.documentElement = newChild;
    
    return newChild;
}

function NodeHasChildNodes() 
{
    if(!this.childNodes)
        return false;

    if(this.childNodes.length > 0)
    {
        return true;
    }
    return false;
}

function NodeCloneNode(deep)
{
    var clone = new this.constructor();
    clone.ownerDocument = this.ownerDocument;

    for(var i in this)
    {
        if(this[i] && (typeof this[i] == 'object') && (this[i].cloneNode != null))
        {
            //noops
            if(i == "parentNode");
            else if(i == "firstChild");
            else if(i == "lastChild");
            else if(i == "previousSibling");
            else if(i == "nextSibling");
            else if(i == "ownerDocument");
            else
            {
                clone[i] = this[i];
            }
        }
        else if(i == "attributes")
        {
            for(var x = 0; x < this.attributes.length; x++)
                clone.attributes.setNamedItem(this.attributes.item(x).cloneNode(true));
        }
        else if((i == "childNodes"))
        {
            if(deep == true)
            {
                for(var x = 0; x < this.childNodes.length; x++)
                    clone.appendChild(this.childNodes.item(x).cloneNode(true));
            }
        }
        else
            clone[i] = this[i];
    }

    return clone;
}

function NodeToString()
{
    var tmp = "<";

    if(this.nodeName)
        tmp += this.nodeName + ", ";
    else
        tmp += "null, ";
    if(this.nodeValue)
        tmp += this.nodeValue + ", ";
    else
        tmp += "null, ";
    if(this.nodeType)
        tmp += this.nodeType + ", ";
    else
        tmp += "null, ";
    if(this.childNodes)
        tmp += this.childNodes.toString() + ", ";
    else
        tmp += "null, ";
    if(this.attributes)
        tmp += this.attributes.toString();
    else
        tmp += "null";

    tmp += ">";
    return tmp;
}

function Node_init(obj)
{
    obj.nodeName = new String();
    obj.nodeValue = new String();
    obj.nodeType = 0;
    obj.parentNode = null;
    obj.childNodes = new NodeList();
    obj.firstChild = null;
    obj.lastChild = null;
    obj.previousSibling = null;
    obj.nextSibling = null;
    obj.attributes = new NamedNodeMap();
    obj.ownerDocument = null;
    obj._allowedChildNodeTypes = [];
}

function Node_checkChildNodeType(newChild)
{
    for(var i = 0; i < this._allowedChildNodeTypes.length; i++)
    {
        if(this._allowedChildNodeTypes[i] == newChild.nodeType)
            return true;
    }
    return false;
}

function Node_findChildIndex(aChild)
{
    for(var i = 0; i < this.childNodes.length; i++)
    {
        if(this.childNodes.item(i) == aChild)
            return i;
    }
    return -1;
}

Node.ELEMENT_NODE                   = 1;
Node.ATTRIBUTE_NODE                 = 2;
Node.TEXT_NODE                      = 3;
Node.CDATA_SECTION_NODE             = 4;
Node.ENTITY_REFERENCE_NODE          = 5;
Node.ENTITY_NODE                    = 6;
Node.PROCESSING_INSTRUCTION_NODE    = 7;
Node.COMMENT_NODE                   = 8;
Node.DOCUMENT_NODE                  = 9;
Node.DOCUMENT_TYPE_NODE             = 10;
Node.DOCUMENT_FRAGMENT_NODE         = 11;
Node.NOTATION_NODE                  = 12;
Node.prototype.insertBefore         = NodeInsertBefore;
Node.prototype.replaceChild         = NodeReplaceChild;
Node.prototype.removeChild          = NodeRemoveChild;
Node.prototype.appendChild          = NodeAppendChild;
Node.prototype.hasChildNodes        = NodeHasChildNodes;
Node.prototype.cloneNode            = NodeCloneNode;
Node.prototype.toString             = NodeToString;
Node._init                          = Node_init;
Node.prototype._checkChildNodeType  = Node_checkChildNodeType;
Node.prototype._findChildIndex      = Node_findChildIndex;

/*----------------------------------------------------------------------------
*
* Class: DocumentFragment
*
* Description:
* A piece of a document
*
* Parameters:
* None
*
* Return:
* None
*
------------------------------------------------------------------------------*/
function DocumentFragment()
{
    Node._init(this);
    this.nodeName = "#document-fragment";
    this.nodeType = Node.DOCUMENT_FRAGMENT_NODE;
    this.attributes = null;
    this.parentNode = null;
    this._allowedChildNodeTypes = [Node.ELEMENT_NODE, 
                                  Node.TEXT_NODE, 
                                  Node.CDATA_SECTION_NODE, 
                                  Node.ENTITY_REFERENCE_NODE, 
                                  Node.PROCESSING_INSTRUCTION_NODE, 
                                  Node.COMMENT_NODE];
}

DocumentFragment.prototype              = new Node();
DocumentFragment.prototype.constructor  = DocumentFragment;

/*----------------------------------------------------------------------------
*
* Class: Document
*
* Description:
* A document
*
* Parameters:
* None
*
* Return:
* None
*
------------------------------------------------------------------------------*/
function Document()
{
    Document_init(this);
}

function DocumentCreateElement(tagName)
{
    var x = new Element();
    x.tagName = x.nodeName = tagName;
    x.ownerDocument = this;
    return x;    
}

function DocumentCreateDocumentFragment()
{
    var x = new DocumentFragment();
    x.ownerDocument = this;
    return x;    
}

function DocumentCreateTextNode(data) 
{
    var x = new Text();
    x.insertData(0, data);
    x.ownerDocument = this;
    return x;    
}

function DocumentCreateComment(data) 
{
    var x = new Comment();
    x.insertData(0, data);
    x.ownerDocument = this;
    return x;    
}

function DocumentCreateCDATASection(data) 
{
    var x = new CDATASection();
    x.insertData(0, data);
    x.ownerDocument = this;
    return x;    
}

function DocumentCreateProcessingInstruction(target, data) 
{
    var x = new ProcessingInstruction();
    x.nodeName = x.target = target;
    x.nodeValue = x.data = data;
    x.ownerDocument = this;
    return x;    
}

function DocumentCreateAttribute(name) 
{
    var x = new Attr();
    x.nodeName = x.name = name;
    x.ownerDocument = this;
    return x;    
}

function DocumentCreateEntityReference(name) 
{
    var x = new EntityReference();
    x.nodeName = name;
    x.ownerDocument = this;
    return x;    
}

function DocumentGetElementsByTagName(tagname) 
{
    return this.documentElement.getElementsByTagName(tagname);
}

function Document_init(obj)
{
    Node._init(obj);
    obj.nodeType = Node.DOCUMENT_NODE;
    obj.doctype = new String();
    obj.implementation = new DOMImplementation();
    obj.documentElement = null;
    obj._allowedChildNodeTypes = [Node.ELEMENT_NODE, 
                                  Node.PROCESSING_INSTRUCTION_NODE, 
                                  Node.COMMENT_NODE, 
                                  Node.DOCUMENT_TYPE_NODE,
                                  Node.TEXT_NODE]
}

Document.prototype                              = new Node();
Document.prototype.constructor                  = Document;
Document.prototype.createElement                = DocumentCreateElement;
Document.prototype.createDocumentFragment       = DocumentCreateDocumentFragment;
Document.prototype.createTextNode               = DocumentCreateTextNode;
Document.prototype.createComment                = DocumentCreateComment;
Document.prototype.createCDATASection           = DocumentCreateCDATASection;
Document.prototype.createProcessingInstruction  = DocumentCreateProcessingInstruction;
Document.prototype.createAttribute              = DocumentCreateAttribute;
Document.prototype.createEntityReference        = DocumentCreateEntityReference;
Document.prototype.getElementsByTagName         = DocumentGetElementsByTagName;
Document._init                                  = Document_init;

/*----------------------------------------------------------------------------
*
* Class: Attr
*
* Description:
* Element Attribute name/value pair
*
* Parameters:
* None
*
* Return:
* None
*
------------------------------------------------------------------------------*/
function Attr()
{
    Node._init(this);
    this.name = this.nodeName;
    this.specified = false;
    this.value = this.nodeValue;
    this.nodeType = Node.ATTRIBUTE_NODE;
    this.attributes = null;
    this._allowedChildNodeTypes = [Node.TEXT_NODE, Node.ENTITY_REFERENCE_NODE];
}

Attr.prototype              = new Node();
Attr.prototype.constructor  = Attr;

/*----------------------------------------------------------------------------
*
* Class: Element
*
* Description:
* An element
*
* Parameters:
* None
*
* Return:
* None
*
------------------------------------------------------------------------------*/
function Element()
{
    Node._init(this);
    this.tagName = this.nodeName;
    this.nodeType = Node.ELEMENT_NODE;
    this._allowedChildNodeTypes = [Node.ELEMENT_NODE, 
                                   Node.TEXT_NODE, 
                                   Node.CDATA_SECTION_NODE, 
                                   Node.ENTITY_REFERENCE_NODE, 
                                   Node.PROCESSING_INSTRUCTION_NODE, 
                                   Node.COMMENT_NODE];
}

function ElementGetAttribute(name)
{
    try
    {
        return this.attributes.getNamedItem(name).nodeValue;
    }
    catch(e)
    {
        return "";
    }
}

function ElementSetAttribute(name, value) 
{
    var tmp;

    tmp = this.getAttribute(name)
    if(tmp == "")
    {
        tmp = this.ownerDocument.createAttribute(name);
    }
    tmp.nodeValue = tmp.value = value;
    this.attributes.setNamedItem(tmp);
}

function ElementRemoveAttribute(name) 
{
    this.attributes.removeNamedItem(name);
}

function ElementGetAttributeNode(name) 
{
    try
    {
        return this.attributes.getNamedItem(name);
    }
    catch(e)
    {
        return null;
    }
}

function ElementSetAttributeNode(newAttr) 
{
    if(newAttr.ownerDocument != this.ownerDocument)
    {
        de = new DOMException();
        de.code = ExceptionCode.WRONG_DOCUMENT_ERR;
        throw(de);
    }

    if(newAttr.parentNode != null)
    {
        de = new DOMException();
        de.code = ExceptionCode.INUSE_ATTRIBUTE_ERR;
        throw(de);
    }

    return this.attributes.setNamedItem(newAttr);
}

function ElementRemoveAttributeNode(oldAttr) 
{
    return this.attributes.removeNamedItem(oldAttr.name);
}

function ElementGetElementsByTagName(name) 
{
    var elements = new NodeList();

    if(name == "*" && this.nodeType == Node.ELEMENT_NODE)
        elements.list.push(this);
    else if(this.tagName == name && this.nodeType == Node.ELEMENT_NODE)
        elements.list.push(this);

    elements.length = elements.list.length;
    return this._findElementsByTagName(name, elements);
}

function ElementNormalize() 
{
    curNode = this.firstChild;
    while(curNode)
    {
        if(curNode.nodeType == Node.ENTITY_REFERENCE_NODE || curNode.nodeType == Node.ELEMENT_NODE)
        {
            curNode.normalize();
        }

        if(curNode && (curNode.nodeType == Node.TEXT_NODE))
        {
            if(curNode.previousSibling && (curNode.previousSibling.nodeType == Node.TEXT_NODE))
            {
                curNode.insertData(0, curNode.previousSibling.substringData(0, curNode.previousSibling.length));
                this.removeChild(curNode.previousSibling);
            }
        }

        if(curNode)
        {
            curNode = curNode.nextSibling;
        }
    }
}

function Element_findElementsByTagName(name, elements)
{
    var i;

    for(i = 0; i < this.childNodes.length; i++)
    {
        curEl = this.childNodes.item(i);
        if(curEl.nodeType == Node.ENTITY_REFERENCE_NODE || curEl.nodeType == Node.ELEMENT_NODE)
        {
            if(name == "*" && curEl.nodeType == Node.ELEMENT_NODE)
                elements.list.push(curEl);
            else if(curEl.tagName == name && curEl.nodeType == Node.ELEMENT_NODE)
                elements.list.push(curEl);
    
            curEl._findElementsByTagName(name, elements);
        }
    }

    elements.length = elements.list.length;
    return elements;
}

Element.prototype                           = new Node();
Element.prototype.constructor               = Element;
Element.prototype.getAttribute              = ElementGetAttribute;
Element.prototype.setAttribute              = ElementSetAttribute;
Element.prototype.removeAttribute           = ElementRemoveAttribute;
Element.prototype.getAttributeNode          = ElementGetAttributeNode;
Element.prototype.setAttributeNode          = ElementSetAttributeNode;
Element.prototype.removeAttributeNode       = ElementRemoveAttributeNode;
Element.prototype.getElementsByTagName      = ElementGetElementsByTagName;
Element.prototype._findElementsByTagName    = Element_findElementsByTagName;
Element.prototype.normalize                 = ElementNormalize;

/*----------------------------------------------------------------------------
*
* Class: DocumentType
*
* Description:
* The type of a document
*
* Parameters:
* None
*
* Return:
* None
*
------------------------------------------------------------------------------*/
function DocumentType()
{
    Node._init(this);
    this.name = this.nodeName;
    this.nodeType = Node.DOCUMENT_TYPE_NODE;
    this.attributes = null;
    this._allowedChildNodeTypes = [];
}

DocumentType.prototype              = new Node();
DocumentType.prototype.Entities     = new NamedNodeMap();
DocumentType.prototype.notations    = new NamedNodeMap();
DocumentType.prototype.constructor  = DocumentType;

/*----------------------------------------------------------------------------
*
* Class: Notation
*
* Description:
* A notation
*
* Parameters:
* None
*
* Return:
* None
*
------------------------------------------------------------------------------*/
function Notation()
{
    Node._init(this);
    this.publicId = new String();
    this.systemId = new String();
    this.nodeType = Node.NOTATION_NODE;
    this.attributes = null;
    this._allowedChildNodeTypes = [];
    this.parentNode = null;
}

Notation.prototype              = new Node();
Notation.prototype.constructor  = Notation;

/*----------------------------------------------------------------------------
*
* Class: Entity
*
* Description:
* An Entity
*
* Parameters:
* None
*
* Return:
* None
*
------------------------------------------------------------------------------*/
function Entity()
{
    Node._init(this);
    this.publicId = new String();
    this.systemId = new String();
    this.notationName = new String();
    this.nodeType = Node.ENTITY_NODE;
    this.attributes = null;
    this._allowedChildNodeTypes = [Node.ELEMENT_NODE, 
                                   Node.TEXT_NODE, 
                                   Node.CDATA_SECTION_NODE, 
                                   Node.ENTITY_REFERENCE_NODE, 
                                   Node.PROCESSING_INSTRUCTION_NODE, 
                                   Node.COMMENT_NODE];
}

Entity.prototype                = new Node();
Entity.prototype.constructor    = Entity;

/*----------------------------------------------------------------------------
*
* Class: EntityReference
*
* Description:
* An EntityReference
*
* Parameters:
* None
*
* Return:
* None
*
------------------------------------------------------------------------------*/
function EntityReference()
{
    Node._init(this);
    this.nodeType = Node.ENTITY_REFERENCE_NODE;
    this.attributes = null;
    this._allowedChildNodeTypes = [Node.ELEMENT_NODE, 
                                   Node.TEXT_NODE, 
                                   Node.CDATA_SECTION_NODE, 
                                   Node.ENTITY_REFERENCE_NODE, 
                                   Node.COMMENT_NODE];
}

//this function is undocumented and only exists to satisfy the case
//where an Element contains an EntityReference which in turn somewhere
//beneath it contains Text nodes that need to be normalized
//the spec is not clear on this
function EntityReferenceNormalize() 
{
    curNode = this.firstChild;
    while(curNode)
    {
        if(curNode.nodeType == Node.ENTITY_REFERENCE_NODE || curNode.nodeType == Node.ELEMENT_NODE)
        {
            curNode.normalize();
        }

        if(curNode && (curNode.nodeType == Node.TEXT_NODE))
        {
            if(curNode.previousSibling && (curNode.previousSibling.nodeType == Node.TEXT_NODE))
            {
                curNode.insertData(0, curNode.previousSibling.substringData(0, curNode.previousSibling.length));
                this.removeChild(curNode.previousSibling);
            }
        }

        if(curNode)
        {
            curNode = curNode.nextSibling;
        }
    }
}

//this private function only exists to satisfy the case
//where an Element contains an EntityReference which in turn somewhere
//beneath it contains Elements.  The spec is not clear on this
function EntityReference_findElementsByTagName(name, elements)
{
    var i;

    for(i = 0; i < this.childNodes.length; i++)
    {
        curEl = this.childNodes.item(i);
        if(curEl.nodeType == Node.ENTITY_REFERENCE_NODE || curEl.nodeType == Node.ELEMENT_NODE)
        {
            if(name == "*" && curEl.nodeType == Node.ELEMENT_NODE)
                elements.list.push(curEl);
            else if(curEl.tagName == name && curEl.nodeType == Node.ELEMENT_NODE)
                elements.list.push(curEl);
        }
        curEl._findElementsByTagName(name, elements);
    }

    elements.length = elements.list.length;
    return elements;
}

EntityReference.prototype                           = new Node();
EntityReference.prototype.constructor               = EntityReference;
EntityReference.prototype.normalize                 = EntityReferenceNormalize;
EntityReference.prototype._findElementsByTagName    = EntityReference_findElementsByTagName;

/*----------------------------------------------------------------------------
*
* Class: ProcessingInstruction
*
* Description:
* A ProcessingInstruction
*
* Parameters:
* None
*
* Return:
* None
*
------------------------------------------------------------------------------*/
function ProcessingInstruction()
{
    Node._init(this);
    this.target = this.nodeName;
    this.data = this.nodeValue;
    this.nodeType = Node.PROCESSING_INSTRUCTION_NODE;
    this.nodeName = this.target;
    this.nodeValue = this.data;
    this.attributes = null;
    this._allowedChildNodeTypes = [];
}

ProcessingInstruction.prototype             = new Node();
ProcessingInstruction.prototype.constructor = ProcessingInstruction;

/*----------------------------------------------------------------------------
*
* Class: CharacterData
*
* Description:
* A blob of characters
*
* Parameters:
* None
*
* Return:
* None
*
------------------------------------------------------------------------------*/
function CharacterData()
{
    CharacterData_init(this);
}

function CharacterDataSubstringData(offset, count)
{
    if((offset < 0) || (offset > this.data.length) || (count < 0))
    {
        de = new DOMException();
        de.code = ExceptionCode.INDEX_SIZE_ERR;
        throw(de);
    }

    if(offset + count > this.data.length)
        count = this.data.length - offset;
            
    return this.data.substring(offset, offset + count);
}

function CharacterDataAppendData(arg) 
{
    if(this.data)
        this.data += arg;
    else
        this.data = arg;
    this.nodeValue = this.data;
    this.length = this.data.length;
}

function CharacterDataInsertData(offset, arg) 
{
    var pre;
    var suf;

    if((offset < 0) || (offset > this.data.length))
    {
        de = new DOMException();
        de.code = ExceptionCode.INDEX_SIZE_ERR;
        throw(de);
    }

    pre = this.data.substring(0, offset);
    suf = this.data.substring(offset, this.data.length);
    this.data = pre + arg + suf;
    this.nodeValue = this.data;
    this.length = this.data.length;
}

function CharacterDataDeleteData(offset, count) 
{
    var pre;
    var suf;

    if((offset < 0) || (offset > this.data.length) || (count < 0))
    {
        de = new DOMException();
        de.code = ExceptionCode.INDEX_SIZE_ERR;
        throw(de);
    }

    pre = this.data.substring(0, offset);
    suf = this.data.substring(offset + count, this.data.length);
    this.data = pre + suf;
    this.nodeValue = this.data;
    this.length = this.data.length;
}

function CharacterDataReplaceData(offset, count, arg) 
{
    this.deleteData(offset, count);
    this.insertData(offset, arg);
}

function CharacterData_init(obj) 
{
    Node._init(obj);
    obj.data = obj.nodeValue;
    obj.length = 0;
    obj.nodeType = null;
    obj.attributes = null;
    obj._allowedChildNodeTypes = [];
}

CharacterData.prototype                 = new Node();
CharacterData.prototype.constructor     = CharacterData;
CharacterData.prototype.substringData   = CharacterDataSubstringData;
CharacterData.prototype.appendData      = CharacterDataAppendData;
CharacterData.prototype.insertData      = CharacterDataInsertData;
CharacterData.prototype.deleteData      = CharacterDataDeleteData;
CharacterData.prototype.replaceData     = CharacterDataReplaceData;
CharacterData._init                     = CharacterData_init;

/*----------------------------------------------------------------------------
*
* Class: Comment
*
* Description:
* An comment
*
* Parameters:
* None
*
* Return:
* None
*
------------------------------------------------------------------------------*/
function Comment()
{
    this.nodeName = "#comment";
    this.nodeType = Node.COMMENT_NODE;
    this.attributes = null;
    this._allowedChildNodeTypes = [];
}

Comment.prototype               = new CharacterData();
Comment.prototype.constructor   = Comment;

/*----------------------------------------------------------------------------
*
* Class: Text
*
* Description:
* A blob of text
*
* Parameters:
* None
*
* Return:
* None
*
------------------------------------------------------------------------------*/
function Text()
{
    Text_init(this);
}

function TextSplitText(offset)
{
    var newData;
    var newNode;

    //spec doesn't mention this err, but has to be
    if(!this.parentNode)
    {
        de = new DOMException();
        de.code = ExceptionCode.HIERARCHY_REQUEST_ERR;
        throw(de);
    }

    if(offset < 0 || offset > this.data.length)
    {
        de = new DOMException();
        de.code = ExceptionCode.INDEX_SIZE_ERR;
        throw(de);
    }

    newData = this.substringData(offset, this.data.length - offset);
    this.deleteData(offset, this.data.length - offset);
    newNode = this.ownerDocument.createTextNode(newData);
    if(this.nextSibling)
        this.parentNode.insertBefore(newNode, this.nextSibling);
    else
        this.parentNode.appendChild(newNode);
}

function Text_init(obj)
{
    CharacterData._init(obj);
    obj.nodeName = "#text";
    obj.nodeType = Node.TEXT_NODE;
    obj.attributes = null;
    obj._allowedChildNodeTypes = [];
}

Text.prototype              = new CharacterData();
Text.prototype.constructor  = Text;
Text.prototype.splitText    = TextSplitText;
Text._init                  = Text_init;

/*----------------------------------------------------------------------------
*
* Class: CDATASection
*
* Description:
* A section of unparsed text
*
* Parameters:
* None
*
* Return:
* None
*
------------------------------------------------------------------------------*/
function CDATASection()
{
    this.nodeName = "#cdata-section"
    this.nodeType = Node.CDATA_SECTION_NODE;
    this.attributes = null;
    this._allowedChildNodeTypes = [];
}

CDATASection.prototype              = new Text();
CDATASection.prototype.constructor  = CDATASection;
