// ============================================================================
//  media.ms
//
//  Useful addons to the media object
//
//  loadAsRgb(...)
//      This function takes all the load arguments plus optional SourceProfile
//      and DestProfile members. If the specified image is already RGB, then
//      no conversion is performed.
//
//  Example:
//      -- convert the media after load to rgb using the specified profile.
//      image = new Media();
//      image.loadAsRgb(name @ "myImage.tif", DestProfile @ "sRGB Color Space Profile.icm");
//
//      -- convert the media after load to rgb using the default profile.
//      image = new Media();
//      image.loadAsRgb(name @ "myImage.tif");
//
// ============================================================================

function Media_loadAsRgb()
{
    var params;

    if (typeof(arguments.length) == "undefined")
    {
        for (var i in arguments)
            params[i] = arguments[i];
    }
    else
    {
        params = arguments[0];
    }

    var imgName = params.name;
    
    var isExcel = false;
    var dotPos = imgName.lastIndexOf(".");

    if (dotPos >= 0)
        isExcel = ("xls" == imgName.substring(dotPos + 1).toLowerCase());

    if (isExcel)
    {
        params.imageHeight = 1000;
        params.imageWidth = 1000;
        params.pages = 1;
    }

    params.useCMYK = false;     // force all PDF/EPS/AI/Office docs to load as RGB so we don't even have to bother with the colorspace conversion for those formats

    var res = this.load(params);
    var format;

    try
    {
        // if there is no format, then paraminfo was specified.
        format = this.getPixelFormat();
    }
    catch(e)
    {
        //  add source and dest to param info and return it
        res += "[DestProfile=<text>]\n" +
                "[Intent=(Perceptual,RelativeColorimetric,Saturation,AbsoluteColorimetric)] default=Perceptual\n" +
                "[OverrideEmbedded]\n" +
                "[ApplyEmbedded]\n" +
                "[SourceProfile=<text>]";
        return res;
    }

    if (format.indexOf("CMYK") == 0)
    {
        var correctParams;

        var foundSource = false;
        var foundDest = false;

        // Copy the caller's colormanage params to the parameter object for the colorCorrect call.
        // The parameters are case insensitive to match the media object
        // convention.
        for (var i in params)
        {
            if (!foundSource && i.search(/sourceprofile/i) == 0)
            {
                correctParams.SourceProfile = params[i];
                foundSource = true;
            }
            else if (!foundDest && i.search(/destprofile/i) == 0)
            {
                correctParams.DestProfile = params[i];
                foundDest = true;
            }
            else if (i.search(/intent/i) == 0)
            {
                correctParams.Intent = params[i];
            }
            else if (i.search(/overrideembedded/i) == 0)
            {
                correctParams.OverrideEmbedded = params[i];
            }
        }

        //  if no sourceprofile specified, use the default
        if (!foundSource)
            correctParams.SourceProfile = "cmyk";

        //  if no destprofile specified, use the default
        if (!foundDest)
            correctParams.DestProfile = "rgb";

        //  correct the image to rgb
        this.colorCorrect(correctParams);
    }
    else if (this.embeddedProfile())
    {
        for (var i in params)
            if (i.search(/applyembedded/i) == 0 && params[i])
                this.colorCorrect(destProfile @ "RGB");
    }
}

Media.prototype.loadAsRgb = Media_loadAsRgb;
Media.prototype.loadAsRGB = Media_loadAsRgb;    // an alias so it matches some places in the docs
